import { Component, h, State, Prop, Event, Watch } from '@stencil/core';
import { Dates } from '../../utils/dates';
import { i18n } from '../../utils/i18n';
/**
 * @since 1.0
 * @status experimental
 *
 * @part base - The component's base wrapper
 */
export class FiscalCalendar {
  constructor() {
    this.daysNamesShort = [];
    this.daysNames = [];
    this.monthNames = [];
    /** The selected date of the calendar, given as a fiscalDate number. */
    this.date = Dates.dateToFiscalDate(new Date());
    /**
     * Defines the systemDate as a fiscalDate. If not set, the current date will automatically be selected.
     */
    this.systemDate = -1;
    for (var i = 0; i < 7; i++) {
      const weekdayIndex = i == 6 ? 0 : i + 1; // Make monday first day of week
      this.daysNamesShort.push(Dates.formatWeekDay(weekdayIndex, 'short'));
      this.daysNames.push(Dates.formatWeekDay(weekdayIndex));
    }
    for (var i = 0; i < 12; i++) {
      this.monthNames.push(Dates.formatMonth(i));
    }
  }
  handleDateChange() {
    this.select(this.date, false);
  }
  componentWillLoad() {
    if (this.systemDate < 0) {
      this.systemDate = Dates.dateToFiscalDate(new Date());
    }
    this.gotoDate(this.systemDate);
  }
  setDates(fdate) {
    var date = fdate;
    if (this.date === date)
      return;
    this.gotoDate(date);
    this.date = date;
  }
  gotoDate(fdate) {
    var date = Dates.fiscalDateToDate(fdate);
    if (!date)
      return;
    this.year = date.getFullYear();
    this.month = date.getMonth();
  }
  getFirstDayOfMonth() {
    return new Date(this.year, this.month, 1).getDay();
  }
  getDaysInMonth() {
    return 32 - new Date(this.year, this.month, 32).getDate();
  }
  getWeekDates(firstDate) {
    var result = [];
    for (var i = 0; i < 7; i++) {
      result.push(firstDate + i);
    }
    return result;
  }
  getWeekdayDates(firstDate, rows) {
    var result = [];
    for (var i = 0; i < rows; i++) {
      let fdate = firstDate + i * 7;
      var date = Dates.fiscalDateToDate(fdate);
      if (date.getMonth() === this.month)
        result.push(fdate);
    }
    return result;
  }
  adjustMonth(offset) {
    var date = new Date(this.year, this.month + offset);
    this.year = date.getFullYear();
    this.month = date.getMonth();
    //this.draw();
  }
  select(fdate, emit = true) {
    this.date = fdate;
    this.gotoDate(fdate);
    if (emit) {
      this.dateChange.emit(this.date);
    }
  }
  selectWeekdayDates(firstDate, rows) {
    const dates = this.getWeekdayDates(firstDate, rows);
    this.select(dates[0]);
  }
  selectWeekDates(fdate) {
    const dates = this.getWeekDates(fdate);
    this.select(dates[0]);
  }
  getWeek(date) {
    var date = new Date(date.getTime());
    date.setHours(0, 0, 0, 0);
    date.setDate(date.getDate() + 3 - ((date.getDay() + 6) % 7));
    var week1 = new Date(date.getFullYear(), 0, 4);
    return 1 + Math.round(((date.getTime() - week1.getTime()) / 86400000 - 3 + ((week1.getDay() + 6) % 7)) / 7);
  }
  render() {
    var firstDay = 1;
    var leadDays = (this.getFirstDayOfMonth() - firstDay + 7) % 7;
    var rows = Math.ceil((leadDays + this.getDaysInMonth()) / 7);
    var fdate = Dates.dateToFiscalDate(new Date(this.year, this.month, 1 - leadDays));
    var ftoday = this.systemDate;
    //let moreBefore = (Math.min.apply(null, this.fdates) < fdate) ? " datepicker-more" : "";
    //let moreAfter = (Math.max.apply(null, this.fdates) >= (fdate + rows * 7)) ? " datepicker-more" : "";
    let dateMatrix = [];
    let currentDate = fdate;
    for (var row = 0; row < rows; row++) {
      dateMatrix.push([]);
      //days
      for (var col = 0; col < 7; col++) {
        dateMatrix[row].push(currentDate);
        currentDate++;
      }
    }
    return (h("div", { class: "datepicker", part: "base" },
      h("div", { class: "datepicker-header" },
        h("div", { class: "datepicker-title" },
          this.monthNames[this.month],
          " ",
          this.year),
        h("div", { class: "datepicker-navigate" },
          h("xena-tooltip", { content: i18n('PREVIOUS_MONTH'), placement: "bottom" },
            h("xena-icon-button", { name: "fas-chevron-left", class: "datepicker-jump", onClick: () => this.adjustMonth(-1), tabindex: "-1" })),
          h("xena-tooltip", { content: i18n('NEXT_MONTH'), placement: "bottom-end" },
            h("xena-icon-button", { name: "fas-chevron-right", class: "datepicker-jump", onClick: () => this.adjustMonth(+1), tabindex: "-1" })))),
      h("table", { class: "datepicker-dates" },
        h("thead", null,
          h("tr", null,
            h("th", { class: "datepicker-week" }),
            this.daysNamesShort.map((day, i) => (h("th", { title: this.daysNames[i], onClick: () => this.selectWeekdayDates(fdate + i, rows) }, day))))),
        h("tbody", null, dateMatrix.map(row => (h("tr", null,
          h("th", { class: "datepicker-week", onClick: () => this.selectWeekDates(row[0]) }, this.getWeek(Dates.fiscalDateToDate(row[0]))),
          row.map(col => (h("td", { class: {
              'datepicker-today': col === ftoday,
              'datepicker-selected': col === this.date,
              'datepicker-other-month': Dates.fiscalDateToDate(col).getMonth() !== this.month
            }, onClick: () => this.select(col) },
            h("div", { class: "date-cell-inner" }, Dates.fiscalDateToDate(col).getDate())))))))))));
  }
  static get is() { return "xena-fiscal-calendar"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["fiscal-calendar.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["fiscal-calendar.css"]
  }; }
  static get properties() { return {
    "date": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The selected date of the calendar, given as a fiscalDate number."
      },
      "attribute": "date",
      "reflect": false,
      "defaultValue": "Dates.dateToFiscalDate(new Date())"
    },
    "systemDate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the systemDate as a fiscalDate. If not set, the current date will automatically be selected."
      },
      "attribute": "system-date",
      "reflect": false,
      "defaultValue": "-1"
    }
  }; }
  static get states() { return {
    "year": {},
    "month": {}
  }; }
  static get events() { return [{
      "method": "dateChange",
      "name": "dateChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits the selected fiscalDate."
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }]; }
  static get watchers() { return [{
      "propName": "date",
      "methodName": "handleDateChange"
    }]; }
}
