import { Component, Element, Event, Method, Prop, State, Watch, h } from '@stencil/core';
import ModalHandler from '../../utils/modal-handler';
import { lockBodyScrolling, unlockBodyScrolling } from '../../utils/scroll';
import { hasSlot } from '../../utils/slot';
let id = 0;
/**
 * @since 1.0
 * @status stable
 *
 * @slot - The modal's content.
 * @slot footer - The modal's footer, usually one or more buttons representing various options.
 *
 * @part base - The component's base wrapper.
 * @part overlay - The overlay.
 * @part panel - The modal panel (where the modal and its is rendered).
 * @part header - The modal header.
 * @part title - The modal title.
 * @part close-button - The close button.
 * @part body - The modal body.
 * @part footer - The modal footer.
 *
 */
export class Modal {
  constructor() {
    this.componentId = `modal-${++id}`;
    this.isShowing = false;
    this.hasFooter = false;
    /** Indicates whether or not the modal is open. You can use this in lieu of the show/hide methods. */
    this.open = false;
    /**
     * The modal's label as displayed in the header. You should always include a relevant label even when using
     * `no-header`, as it is required for proper accessibility.
     */
    this.label = '';
    /**
     * Set to true to disable the header. This will also remove the default close button, so please ensure you provide an
     * easy, accessible way for users to dismiss the modal.
     */
    this.noHeader = false;
    /**
     * Set to true to remove the padding in the boddy.
     */
    this.noBodyPadding = false;
    /**
     * Set to true to show a default close button in the header of the dialog.
     */
    this.closable = false;
    /**
     * Set to true to let clicks on the overlay close the dialog.
     */
    this.closableOverlay = false;
  }
  handleOpenChange() {
    this.open ? this.show() : this.hide();
  }
  connectedCallback() {
    this.handleCloseClick = this.handleCloseClick.bind(this);
    this.handleTransitionEnd = this.handleTransitionEnd.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
    this.handleOverlayClick = this.handleOverlayClick.bind(this);
    this.updateSlots = this.updateSlots.bind(this);
    this.modalHandler = new ModalHandler(this.host, {
      onFocusOut: () => this.panel.focus()
    });
  }
  componentWillLoad() {
    this.updateSlots();
    this.host.shadowRoot.addEventListener('slotchange', this.updateSlots);
  }
  componentDidLoad() {
    // Show on init if open
    if (this.open) {
      this.show();
    }
  }
  disconnectedCallback() {
    unlockBodyScrolling(this.host);
    this.host.shadowRoot.removeEventListener('slotchange', this.updateSlots);
  }
  /** Shows the modal */
  async show() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (this.isShowing) {
      return;
    }
    const xnShow = this.xnShow.emit();
    if (xnShow.defaultPrevented) {
      this.open = false;
      return;
    }
    this.modal.hidden = false;
    this.host.clientWidth; // force a reflow
    this.isShowing = true;
    this.open = true;
    this.modalHandler.activate();
    lockBodyScrolling(this.host);
  }
  /** Hides the modal */
  async hide() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (!this.isShowing) {
      return;
    }
    const xnHide = this.xnHide.emit();
    if (xnHide.defaultPrevented) {
      this.open = true;
      return;
    }
    this.isShowing = false;
    this.open = false;
    this.modalHandler.deactivate();
    unlockBodyScrolling(this.host);
  }
  handleCloseClick() {
    this.hide();
  }
  handleKeyDown(event) {
    if (event.key === 'Escape') {
      this.hide();
    }
  }
  handleOverlayClick() {
    const xnOverlayDismiss = this.xnOverlayDismiss.emit();
    if (!xnOverlayDismiss.defaultPrevented && this.closableOverlay) {
      this.hide();
    }
  }
  handleTransitionEnd(event) {
    const target = event.target;
    // Ensure we only emit one event when the target element is no longer visible
    if (event.propertyName === 'opacity' && target.classList.contains('modal__panel')) {
      this.modal.hidden = !this.open;
      this.open ? this.xnAfterShow.emit() : this.xnAfterHide.emit();
    }
  }
  updateSlots() {
    this.hasFooter = hasSlot(this.host, 'footer');
  }
  render() {
    return (h("div", { ref: el => (this.modal = el), part: "base", class: {
        modal: true,
        'modal--open': this.open,
        'modal--has-footer': this.hasFooter
      }, onKeyDown: this.handleKeyDown, onTransitionEnd: this.handleTransitionEnd, hidden: true },
      h("div", { part: "overlay", class: "modal__overlay", onClick: this.handleOverlayClick }),
      h("div", { ref: el => (this.panel = el), part: "panel", class: "modal__panel", role: "modal", "aria-modal": "true", "aria-hidden": !this.open, "aria-label": this.noHeader ? this.label : null, "aria-labelledby": !this.noHeader ? `${this.componentId}-title` : null, tabIndex: 0 },
        !this.noHeader && (h("header", { part: "header", class: "modal__header" },
          h("span", { part: "title", class: "modal__title", id: `${this.componentId}-title` }, this.label || String.fromCharCode(65279)),
          this.closable ? (h("div", { part: "close-button", class: "modal__close", onClick: this.handleCloseClick }, "\uD83D\uDDD9")) : (''))),
        h("div", { part: "body", class: {
            modal__body: true,
            'modal__no-padding': this.noBodyPadding
          } },
          h("slot", null)),
        h("footer", { part: "footer", class: "modal__footer" },
          h("slot", { name: "footer" })))));
  }
  static get is() { return "xena-modal"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["modal.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["modal.css"]
  }; }
  static get properties() { return {
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates whether or not the modal is open. You can use this in lieu of the show/hide methods."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The modal's label as displayed in the header. You should always include a relevant label even when using\n`no-header`, as it is required for proper accessibility."
      },
      "attribute": "label",
      "reflect": false,
      "defaultValue": "''"
    },
    "noHeader": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to disable the header. This will also remove the default close button, so please ensure you provide an\neasy, accessible way for users to dismiss the modal."
      },
      "attribute": "no-header",
      "reflect": false,
      "defaultValue": "false"
    },
    "noBodyPadding": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to remove the padding in the boddy."
      },
      "attribute": "no-body-padding",
      "reflect": false,
      "defaultValue": "false"
    },
    "closable": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to show a default close button in the header of the dialog."
      },
      "attribute": "closable",
      "reflect": false,
      "defaultValue": "false"
    },
    "closableOverlay": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to let clicks on the overlay close the dialog."
      },
      "attribute": "closable-overlay",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "hasFooter": {}
  }; }
  static get events() { return [{
      "method": "xnShow",
      "name": "xnShow",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the modal opens. Calling `event.preventDefault()` will prevent it from being opened."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnAfterShow",
      "name": "xnAfterShow",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the modal opens and all transitions are complete."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnHide",
      "name": "xnHide",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the modal closes. Calling `event.preventDefault()` will prevent it from being closed."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnAfterHide",
      "name": "xnAfterHide",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the modal closes and all transitions are complete."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnOverlayDismiss",
      "name": "xnOverlayDismiss",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the overlay is clicked. Calling `event.preventDefault()` will prevent the modal from closing."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "show": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Shows the modal",
        "tags": []
      }
    },
    "hide": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Hides the modal",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "open",
      "methodName": "handleOpenChange"
    }]; }
}
