import { Component, Event, h, Prop, Element, Watch } from '@stencil/core';
/**
 * @since 1.8
 * @status experimental
 *
 * @slot Used to set radio components inside the picker. Only xena-radio components are allowed.
 *
 * @part base - The component's base wrapper.
 */
export class ColorPicker {
  constructor() {
    /** Set to true to disable the component. */
    this.disabled = false;
  }
  onValueChanged() {
    this.valueChange.emit(this.value);
    this.syncItemsFromValue();
  }
  componentDidLoad() {
    this.radioGroup.addEventListener('checkedChange', ev => this.handleCheckedChange(ev));
    this.syncItemsFromValue();
  }
  syncItemsFromValue() {
    const items = this.getItems();
    const value = this.value;
    // Sync checked states
    items.map(item => (item.checked = item.value === value));
  }
  getItems() {
    if (this.radioGroup == null)
      return [];
    const slot = this.radioGroup.querySelector('slot');
    const otherThanSlot = this.radioGroup.children;
    return [...otherThanSlot, ...slot.assignedElements({ flatten: true })].filter((el) => el.tagName.toLowerCase() === 'xena-radio' && !el.disabled);
  }
  handleCheckedChange(ev) {
    const target = ev.target;
    if (target.checked) {
      this.value = target.value;
    }
  }
  handleSlotChange() {
    this.syncItemsFromValue();
    this.reportDuplicateItemValues();
  }
  reportDuplicateItemValues() {
    const items = this.getItems();
    // Report duplicate values
    const duplicateValues = items.map(item => item.value).filter((e, i, a) => a.indexOf(e) !== i);
    if (duplicateValues.length) {
      throw new Error('Duplicate value found on <xena-radio> in <xena-radio-group>: "' + duplicateValues.join('", "') + '"');
    }
  }
  render() {
    return (h("div", { class: "radio-group", ref: el => (this.radioGroup = el) },
      h("slot", { onSlotchange: _ => this.handleSlotChange() })));
  }
  static get is() { return "xena-radio-group"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["radio-group.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["radio-group.css"]
  }; }
  static get properties() { return {
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set to true to disable the component."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value of this component."
      },
      "attribute": "value",
      "reflect": true
    }
  }; }
  static get events() { return [{
      "method": "valueChange",
      "name": "valueChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emits any changes to the value of the component."
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "onValueChanged"
    }]; }
}
