import { Component, h, Prop, Element, Event, Watch } from '@stencil/core';
/**
 * @since 1.8
 * @status experimental
 *
 * @part base - The component's base wrapper.
 * @part container - The slider's container.
 * @part handler - The slider's handler.
 */
export class Slider {
  constructor() {
    /** The components value is split into 'x' and 'y'. Default values are 0. */
    this.value = { x: 0, y: 0 };
    /** The property for setting the slider's minimum value. Default is 0. */
    this.min = { x: 0, y: 0 };
    /** The property for setting the slider's maximum value. Default is 100. */
    this.max = { x: 100, y: 100 };
    /** The slider type. Default is horizontal. */
    this.type = 'horizontal';
    /** Container height is set with the height-property. It is set with pixels or percentage of containing element. */
    this.height = '';
    /** Container width is set with the width-property. It is set with pixels or percentage of containing element. */
    this.width = '';
    this._value = { x: 0, y: 0 };
    this._min = { x: 0, y: 0 };
    this._max = { x: 100, y: 100 };
    this.isDragging = false;
  }
  valueUpdated() {
    this._value = this.parseInput(this.value);
    this.setHandlerPosition();
  }
  minUpdated() {
    this._min = this.parseInput(this.min);
  }
  maxUpdated() {
    this._max = this.parseInput(this.max);
  }
  parseInput(value) {
    if (typeof value === 'string') {
      const values = value.split(',').map(parseFloat);
      const [x, y = x] = values;
      return { x, y };
    }
    if (typeof value === 'number') {
      return { x: value, y: value };
    }
    return value;
  }
  componentWillLoad() {
    this._min = this.parseInput(this.min);
    this._max = this.parseInput(this.max);
    this._value = this.parseInput(this.value);
  }
  componentDidLoad() {
    this.handler = this.element.shadowRoot.querySelector('.handler');
    this.container = this.element.shadowRoot.querySelector('.container');
    this.handler.addEventListener('mousedown', this.handleMouseDown.bind(this));
    this.container.addEventListener('mousedown', this.handleMouseDown.bind(this));
    document.addEventListener('mousemove', this.handleMouseMove.bind(this));
    document.addEventListener('mouseup', this.handleMouseUp.bind(this));
    document.addEventListener('selectstart', this.handleSelectStart.bind(this));
    this.setHandlerPosition();
  }
  handleMouseDown(e) {
    this.isDragging = true;
    const handlerRect = this.handler;
    const containerRect = this.container;
    const offsetX = e.clientX -
      handlerRect.getBoundingClientRect().left -
      this.handler.clientWidth / 2 -
      containerRect.getBoundingClientRect().width;
    const offsetY = handlerRect.getBoundingClientRect().bottom -
      e.clientY -
      this.handler.clientHeight / 2 -
      (containerRect.getBoundingClientRect().height - handlerRect.getBoundingClientRect().height);
    const maxX = containerRect.getBoundingClientRect().width - handlerRect.getBoundingClientRect().width;
    const maxY = containerRect.getBoundingClientRect().height - handlerRect.getBoundingClientRect().height;
    const clampedOffsetX = Math.min(Math.max(0, offsetX), maxX);
    const clampedOffsetY = Math.min(Math.max(0, offsetY), maxY);
    let newX = e.clientX - containerRect.getBoundingClientRect().left - clampedOffsetX;
    let newY = containerRect.getBoundingClientRect().bottom - e.clientY - clampedOffsetY;
    if (this.type === 'horizontal') {
      newY = 0;
    }
    else if (this.type === 'vertical') {
      newX = 0;
    }
    this.handler.style.left = `${Math.max(0, Math.min(containerRect.clientWidth, newX))}px`;
    this.handler.style.top = `${Math.max(0, Math.min(containerRect.clientHeight, newY))}px`;
    const horizontalPercentage = (parseFloat(this.handler.style.left) / this.container.clientWidth) * 100;
    const verticalPercentage = (parseFloat(this.handler.style.top) / this.container.clientHeight) * 100;
    const x = (horizontalPercentage / 100) * (this._max.x - this._min.x) + this._min.x;
    const y = (verticalPercentage / 100) * (this._max.y - this._min.y) + this._min.y;
    if (this.type === 'horizontal') {
      this._value = { x, y: 0 };
    }
    else if (this.type === 'vertical') {
      this._value = { x: 0, y };
    }
    else if (this.type === 'both') {
      this._value = { x, y };
    }
    this.value = this._value;
    this.valueChange.emit(this._value);
    e.preventDefault();
  }
  handleMouseMove(e) {
    if (!this.isDragging)
      return;
    const handler = this.handler;
    const container = this.container;
    let newX = e.clientX - container.getBoundingClientRect().left;
    let newY = container.getBoundingClientRect().bottom - e.clientY;
    if (this.type === 'horizontal') {
      newY = 0;
    }
    else if (this.type === 'vertical') {
      newX = 0;
    }
    handler.style.left = `${Math.max(0, Math.min(container.clientWidth, newX))}px`;
    handler.style.top = `${Math.max(0, Math.min(container.clientHeight, newY))}px`;
    const horizontalPercentage = (parseFloat(this.handler.style.left) / this.container.clientWidth) * 100;
    const verticalPercentage = (parseFloat(this.handler.style.top) / this.container.clientHeight) * 100;
    const x = (horizontalPercentage / 100) * (this._max.x - this._min.x) + this._min.x;
    const y = (verticalPercentage / 100) * (this._max.y - this._min.y) + this._min.y;
    if (this.type === 'horizontal') {
      this._value = { x, y: 0 };
    }
    else if (this.type === 'vertical') {
      this._value = { x: 0, y };
    }
    else if (this.type === 'both') {
      this._value = { x, y };
    }
    this.value = this._value;
    this.valueChange.emit(this._value);
  }
  handleMouseUp() {
    this.isDragging = false;
  }
  handleSelectStart(e) {
    if (this.isDragging) {
      e.preventDefault();
    }
  }
  setHandlerPosition() {
    const container = this.element.shadowRoot.querySelector('.container');
    const handler = this.element.shadowRoot.querySelector('.handler');
    const positionX = ((this._value.x - this._min.x) / (this._max.x - this._min.x)) * container.clientWidth;
    const positionY = container.clientHeight - ((this._value.y - this._min.y) / (this._max.y - this._min.y)) * container.clientHeight;
    if (this.type === 'horizontal') {
      handler.style.left = `${positionX}px`;
    }
    else if (this.type === 'vertical') {
      handler.style.top = `${positionY}px`;
    }
    else if (this.type === 'both') {
      handler.style.left = `${positionX}px`;
      handler.style.top = `${positionY}px`;
    }
  }
  setWidth() {
    if (this.type === 'horizontal' && !this.width) {
      return '100%';
    }
    else if (this.type === 'both' && !this.width) {
      return '240px';
    }
    else {
      return this.width;
    }
  }
  setHeight() {
    if (this.type === 'vertical' && !this.height) {
      return '34px';
    }
    else if (this.type === 'both' && !this.height) {
      return '160px';
    }
    else {
      return this.height;
    }
  }
  render() {
    return (h("div", { part: "container", style: {
        width: this.setWidth(),
        height: this.setHeight()
      }, class: `container
          ${this.type === 'horizontal' ? 'container-horizontal' : ''}
          ${this.type === 'vertical' ? 'container-vertical' : ''}
          ${this.type === 'both' ? 'container-both' : ''}
          ` },
      h("div", { part: "handler", class: "handler", onMouseDown: event => this.handleMouseDown.bind(event) })));
  }
  static get is() { return "xena-slider"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["slider.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["slider.css"]
  }; }
  static get properties() { return {
    "value": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "string | number | { x: number; y: number }",
        "resolved": "number | string | { x: number; y: number; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The components value is split into 'x' and 'y'. Default values are 0."
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "{ x: 0, y: 0 }"
    },
    "min": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "string | number | { x: number; y: number }",
        "resolved": "number | string | { x: number; y: number; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The property for setting the slider's minimum value. Default is 0."
      },
      "attribute": "min",
      "reflect": true,
      "defaultValue": "{ x: 0, y: 0 }"
    },
    "max": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "string | number | { x: number; y: number }",
        "resolved": "number | string | { x: number; y: number; }",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The property for setting the slider's maximum value. Default is 100."
      },
      "attribute": "max",
      "reflect": true,
      "defaultValue": "{ x: 100, y: 100 }"
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'horizontal' | 'vertical' | 'both'",
        "resolved": "\"both\" | \"horizontal\" | \"vertical\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The slider type. Default is horizontal."
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'horizontal'"
    },
    "height": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Container height is set with the height-property. It is set with pixels or percentage of containing element."
      },
      "attribute": "height",
      "reflect": false,
      "defaultValue": "''"
    },
    "width": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Container width is set with the width-property. It is set with pixels or percentage of containing element."
      },
      "attribute": "width",
      "reflect": false,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "valueChange",
      "name": "valueChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Value is emitted when the handler is moved via mousedown or mousemove events."
      },
      "complexType": {
        "original": "{ x: number; y: number }",
        "resolved": "{ x: number; y: number; }",
        "references": {}
      }
    }]; }
  static get elementRef() { return "element"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "valueUpdated"
    }, {
      "propName": "min",
      "methodName": "minUpdated"
    }, {
      "propName": "max",
      "methodName": "maxUpdated"
    }]; }
}
