import { r as registerInstance, c as createEvent, h, g as getElement } from './index-9032f50f.js';
import { u as unlockBodyScrolling, l as lockBodyScrolling } from './scroll-c3a2dd95.js';
import { h as hasSlot } from './slot-0863a9cc.js';

let activeModals = [];
class ModalFocusHandler {
  constructor(element, options) {
    this.element = element;
    this.options = options;
    this.handleFocusIn = this.handleFocusIn.bind(this);
  }
  activate() {
    activeModals.push(this.element);
    document.addEventListener('focusin', this.handleFocusIn);
  }
  deactivate() {
    activeModals = activeModals.filter(modal => modal !== this.element);
    document.removeEventListener('focusin', this.handleFocusIn);
  }
  isActive() {
    // The "active" modal is always the most recent one shown
    return activeModals[activeModals.length - 1] === this.element;
  }
  handleFocusIn(event) {
    const target = event.target;
    const tagName = this.element.tagName.toLowerCase();
    // If focus is lost while the modal is active, run the onFocusOut callback
    if (this.isActive() && target.closest(tagName) !== this.element && typeof this.options.onFocusOut === 'function') {
      this.options.onFocusOut(event);
    }
  }
}

const modalCss = ":host{position:relative;box-sizing:border-box}:host *,:host *:before,:host *:after{box-sizing:inherit}:host{--width:31rem;display:contents}.modal{display:flex;align-items:center;justify-content:center;justify-content:flex-start;flex-direction:column;margin-top:var(--xena-spacing-4x-large);position:fixed;top:0;right:0;bottom:0;left:0;z-index:var(--xena-z-index-modal)}.modal[hidden]{display:none}.modal__panel{display:flex;flex-direction:column;z-index:2;width:var(--width);max-width:calc(100% - var(--xena-spacing-2x-large));max-height:calc(100% - var(--xena-spacing-2x-large));background-color:var(--xena-panel-background-color);border-radius:var(--xena-border-radius-medium);box-shadow:var(--xena-shadow-large);opacity:0;transform:scale(0.8);transition:var(--xena-transition-medium) opacity, var(--xena-transition-medium) transform}.modal__panel:focus{outline:none}@media screen and (max-width: 420px){.modal__panel{max-height:80vh}}.modal--open .modal__panel{display:flex;opacity:1;transform:none}.modal__header{flex:0 0 auto;display:flex;border-width:var(--xena-input-border-width);border-style:solid;border-color:var(--xena-color-gray-100);background-color:var(--xena-color-gray-100);border-radius:var(--xena-border-radius-medium) var(--xena-border-radius-medium) 0 0}.modal__title{flex:1 1 auto;font-size:var(--xena-font-size-medium);font-weight:var(--xena-font-weight-semibold);line-height:var(--xena-line-height-dense);padding:var(--xena-spacing-small) var(--xena-spacing-medium)}.modal__close{flex:0 0 auto;display:flex;align-items:center;font-size:var(--xena-font-size-x-large);padding:0 var(--xena-spacing-medium);font-size:var(--xena-font-size-medium);cursor:pointer}.modal__body{flex:1 1 auto;padding:var(--xena-spacing-medium);overflow:auto;-webkit-overflow-scrolling:touch}.modal__footer{flex:0 0 auto;text-align:right;padding:var(--xena-spacing-medium);border-top:var(--xena-input-border-width) solid var(--xena-color-gray-100)}.modal__footer ::slotted(xena-button:not(:first-of-type)){margin-left:var(--xena-spacing-x-small)}.modal:not(.modal--has-footer) .modal__footer{display:none}.modal__overlay{position:fixed;top:0;right:0;bottom:0;left:0;background-color:var(--xena-overlay-background-color);opacity:0;transition:var(--xena-transition-medium) opacity}.modal--open .modal__overlay{opacity:1}.modal__body.modal__no-padding{padding:0}";

let id = 0;
let Modal = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.xnShow = createEvent(this, "xnShow", 7);
    this.xnAfterShow = createEvent(this, "xnAfterShow", 7);
    this.xnHide = createEvent(this, "xnHide", 7);
    this.xnAfterHide = createEvent(this, "xnAfterHide", 7);
    this.xnOverlayDismiss = createEvent(this, "xnOverlayDismiss", 7);
    this.componentId = `modal-${++id}`;
    this.isShowing = false;
    this.hasFooter = false;
    /** Indicates whether or not the modal is open. You can use this in lieu of the show/hide methods. */
    this.open = false;
    /**
     * The modal's label as displayed in the header. You should always include a relevant label even when using
     * `no-header`, as it is required for proper accessibility.
     */
    this.label = '';
    /**
     * Set to true to disable the header. This will also remove the default close button, so please ensure you provide an
     * easy, accessible way for users to dismiss the modal.
     */
    this.noHeader = false;
    /**
     * Set to true to remove the padding in the boddy.
     */
    this.noBodyPadding = false;
    /**
     * Set to true to show a default close button in the header of the dialog.
     */
    this.closable = false;
    /**
     * Set to true to let clicks on the overlay close the dialog.
     */
    this.closableOverlay = false;
  }
  handleOpenChange() {
    this.open ? this.show() : this.hide();
  }
  connectedCallback() {
    this.handleCloseClick = this.handleCloseClick.bind(this);
    this.handleTransitionEnd = this.handleTransitionEnd.bind(this);
    this.handleKeyDown = this.handleKeyDown.bind(this);
    this.handleOverlayClick = this.handleOverlayClick.bind(this);
    this.updateSlots = this.updateSlots.bind(this);
    this.modalHandler = new ModalFocusHandler(this.host, {
      onFocusOut: () => this.panel.focus()
    });
  }
  componentWillLoad() {
    this.updateSlots();
    this.host.shadowRoot.addEventListener('slotchange', this.updateSlots);
  }
  componentDidLoad() {
    // Show on init if open
    if (this.open) {
      this.show();
    }
  }
  disconnectedCallback() {
    unlockBodyScrolling(this.host);
    this.host.shadowRoot.removeEventListener('slotchange', this.updateSlots);
  }
  /** Shows the modal */
  async show() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (this.isShowing) {
      return;
    }
    const xnShow = this.xnShow.emit();
    if (xnShow.defaultPrevented) {
      this.open = false;
      return;
    }
    this.modal.hidden = false;
    this.isShowing = true;
    this.open = true;
    this.modalHandler.activate();
    lockBodyScrolling(this.host);
  }
  /** Hides the modal */
  async hide() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (!this.isShowing) {
      return;
    }
    const xnHide = this.xnHide.emit();
    if (xnHide.defaultPrevented) {
      this.open = true;
      return;
    }
    this.isShowing = false;
    this.open = false;
    this.modalHandler.deactivate();
    unlockBodyScrolling(this.host);
  }
  handleCloseClick() {
    this.hide();
  }
  handleKeyDown(event) {
    if (event.key === 'Escape') {
      this.hide();
    }
  }
  handleOverlayClick() {
    const xnOverlayDismiss = this.xnOverlayDismiss.emit();
    if (!xnOverlayDismiss.defaultPrevented && this.closableOverlay) {
      this.hide();
    }
  }
  handleTransitionEnd(event) {
    const target = event.target;
    // Ensure we only emit one event when the target element is no longer visible
    if (event.propertyName === 'opacity' && target.classList.contains('modal__panel')) {
      this.modal.hidden = !this.open;
      this.open ? this.xnAfterShow.emit() : this.xnAfterHide.emit();
    }
  }
  updateSlots() {
    this.hasFooter = hasSlot(this.host, 'footer');
  }
  render() {
    return (h("div", { ref: el => (this.modal = el), part: "base", class: {
        modal: true,
        'modal--open': this.open,
        'modal--has-footer': this.hasFooter
      }, onKeyDown: this.handleKeyDown, onTransitionEnd: this.handleTransitionEnd, hidden: true }, h("div", { part: "overlay", class: "modal__overlay", onClick: this.handleOverlayClick }), h("div", { ref: el => (this.panel = el), part: "panel", class: "modal__panel", role: "modal", "aria-modal": "true", "aria-hidden": !this.open, "aria-label": this.noHeader ? this.label : null, "aria-labelledby": !this.noHeader ? `${this.componentId}-title` : null, tabIndex: 0 }, !this.noHeader && (h("header", { part: "header", class: "modal__header" }, h("span", { part: "title", class: "modal__title", id: `${this.componentId}-title` }, this.label || String.fromCharCode(65279)), this.closable ? (h("div", { part: "close-button", class: "modal__close", onClick: this.handleCloseClick }, "\uD83D\uDDD9")) : (''))), h("div", { part: "body", class: {
        modal__body: true,
        'modal__no-padding': this.noBodyPadding
      } }, h("slot", null)), h("footer", { part: "footer", class: "modal__footer" }, h("slot", { name: "footer" })))));
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "open": ["handleOpenChange"]
  }; }
};
Modal.style = modalCss;

export { Modal as xena_modal };
