import { EventEmitter } from '../../stencil-public-runtime';
import { ShapeTypes, SizeTypes } from '../../utils/types';
/**
 * @since 1.0
 * @status stable
 *
 * @slot prefix - Used to prepend an icon or text element to the autocompletes input field.
 * @slot suffix - Used to append an icon or text element to the autocompletes input field.
 *
 * @part prefix - The prefix container.
 * @part suffix - The suffix container.
 */
export declare class Autocomplete {
  dropdown: HTMLXenaDropdownElement;
  input: HTMLXenaInputElement;
  list: HTMLXenaAutocompleteListElement;
  focused: boolean;
  private host;
  hasPrefixSlot: boolean;
  hasSuffixSlot: boolean;
  hasErrorTextSlot: boolean;
  private mutationObserver;
  _datasource: (searchString: string) => Promise<any[]>;
  _text: string;
  _entries: any[];
  _debounceInput: (ev: CustomEvent) => void;
  /**
   * Set the amount of time, in milliseconds, to wait before initiating search after each keystroke.
   */
  debounce: number;
  /**
   * If `true`, the user cannot interact with the input.
   */
  disabled: boolean;
  /**
   * Specified the max height of the dropdown.
   */
  maxDropdownHeight: string;
  /**
   * The text in dropdown when no results are found.
   */
  noResultText: string;
  /**
   * The property name for the id of the item.
   */
  idProperty: string;
  /** The autocomplete's label. */
  label: string;
  /** The input's help text.  */
  helpText: string;
  /** The input's error text. Alternatively, you can use the error-text slot. */
  errorText: string;
  /**
   * The property name for the label of the item shown in the dropdown list.
   */
  listLabelProperty: string | ((e: any) => string);
  /**
   * The property name for the icon of the item shown in the dropdown list. The name of the icon must be for an icon in the default icon set. (See xena-icon)
   */
  listIconProperty: string | ((e: any) => string);
  /** Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods. */
  open: boolean;
  /**
   * If `true`, the user must fill in a value before submitting a form.
   */
  required: boolean;
  /**
   * The text to show in the input field when user is not searching.
   */
  placeholder: string;
  /**
   * Defines the shape of the input field.
   */
  shape: ShapeTypes;
  /**
   * Defines the size of the input field.
   */
  size: SizeTypes;
  /**
   * The selected value of the component.
   */
  value: any;
  /**
   * The property name for the label of the item shown in the input field.
   */
  valueLabelProperty: string | ((e: any) => string);
  /**
   * Whether the dropdown with results should be shown even when there are no results on the components initial focus.
   */
  showInitialNoResult: boolean;
  protected debounceChanged(): void;
  /**
   * Emitted when value has been changed - either by selecting an item from the list or by deleting the selected value.
   */
  valueChange: EventEmitter;
  /**
   * Sets the datasource to retrieve data from. The datasource is a lambda function which can expect a search query as an argument.
   * In return it must provide a Promise that can deliver an array of data entries.
   * @param datasource
   */
  setDatasource(datasource: (searchString: string) => Promise<any[]>): Promise<void>;
  /**
   * Sets focus on the input field.
   */
  setFocus(): Promise<void>;
  /**
   * Retrieves the native input element.
   */
  getInputElement(): Promise<HTMLInputElement>;
  /**
   * Closes the autocomplete popup if open.
   */
  hide(): Promise<void>;
  /**
   * Opens the autocomplete popup if closed.
   */
  show(): Promise<void>;
  componentWillLoad(): void;
  componentDidRender(): void;
  connectedCallback(): void;
  disconnectedCallback(): void;
  valueChanged(): void;
  handleSlotChange(): void;
  private search;
  private handleChange;
  private handleSelection;
  private handleFocus;
  private handleBlur;
  private handleDropdownShow;
  private updateDefaultActiveIndex;
  private handleDropdownAfterHide;
  private checkIfCleared;
  private resolveId;
  private resolveValueLabel;
  private handleKeyDown;
  private handleValueChanged;
  render(): any;
}
