# Alert

[component-header:xena-alert]

Alerts are used to display important messages either inline or as toast notifications.

```html preview
<xena-alert open>
  <xena-icon slot="icon" name="fas-info-circle"></xena-icon>
  This is a standard alert. You can customize its content and even the icon.
</xena-alert>
```

?> Alerts will not be visible if the `open` attribute is not present.

## Examples

### Types

Set the `type` attribute to change the alert's type.

```html preview
<xena-alert type="primary" header="This is super informative" open>
  <xena-icon slot="icon" name="fas-info-circle"></xena-icon>
  You can tell by how pretty the alert is.
</xena-alert>

<br />

<xena-alert type="success" header="Your changes have been saved" open>
  <xena-icon slot="icon" name="fas-check-circle"></xena-icon>
  You can safely exit the app now.
</xena-alert>

<br />

<xena-alert type="info" header="Your settings have been updated" open>
  <xena-icon slot="icon" name="fas-cog"></xena-icon>
  Settings will take affect on next login.
</xena-alert>

<br />

<xena-alert type="warning" header="Your session has ended" open>
  <xena-icon slot="icon" name="fas-exclamation-triangle"></xena-icon>
  Please login again to continue.
</xena-alert>

<br />

<xena-alert type="danger" header="Your account has been deleted" open>
  <xena-icon slot="icon" name="fas-exclamation-circle"></xena-icon>
  We're very sorry to see you go!
</xena-alert>
```

### Closable

Add the `closable` attribute to show a close button that will hide the alert.

```html preview
<xena-alert type="primary" open closable class="alert-closable">
  <xena-icon slot="icon" name="fas-info-circle"></xena-icon>
  You can close this alert any time!
</xena-alert>

<script>
  const alert = document.querySelector('.alert-closable');
  alert.addEventListener('xena-after-hide', () => {
    setTimeout(() => (alert.open = true), 2000);
  });
</script>
```

### Without Icons

Icons are optional. Simply omit the `icon` slot if you don't want them.

```html preview
<xena-alert type="primary" open> Nothing fancy here, just a simple alert. </xena-alert>
```

### Inline

Add the `inline` attribute to show the alert as an inline alert.

```html preview
<xena-alert type="success" header="Your changes have been saved" open inline>
  <xena-icon slot="icon" name="fas-check-circle"></xena-icon>
  You can safely exit the app now.
</xena-alert>

<br />

<xena-alert type="danger" header="Your account has been deleted" open inline>
  <xena-icon slot="icon" name="fas-exclamation-circle"></xena-icon>
  We're very sorry to see you go!
</xena-alert>

<br />

<xena-alert type="primary" header="Maybe without an icon" open inline> That looks like this. </xena-alert>

<br />

<xena-alert type="primary" open inline> Or without a header. Ba-dum-tch! </xena-alert>
```

### With Action

You can add call to actions by adding a button with the 'action' slot.

```html preview
<xena-alert type="primary" open header="This is noteworthy">
  <xena-icon slot="icon" name="fas-info-circle"></xena-icon>
  <xena-button slot="action" type="default">Get burgers!</xena-button>
  I am grumpy when I am hungry.
</xena-alert>
```

### Duration

Set the `duration` prop to automatically hide an alert after a period of time. This is useful for alerts that don't require acknowledgement.

```html preview
<div class="alert-duration">
  <xena-button type="primary">Show Alert</xena-button>

  <xena-alert type="primary" duration="3000" closable>
    <xena-icon slot="icon" name="info-circle"></xena-icon>
    This alert will automatically hide itself after three seconds, unless you interact with it.
  </xena-alert>
</div>

<script>
  const container = document.querySelector('.alert-duration');
  const button = container.querySelector('xena-button');
  const alert = container.querySelector('xena-alert');

  button.addEventListener('click', () => {
    alert.show();
  });
</script>

<style>
  .alert-duration xena-alert {
    margin-top: var(--xena-spacing-medium);
  }
</style>
```

### Alert and Confirm Dialogs

For convenience, you can display the alert component in a dialog and either alert the user or ask the user to confirm the message.

#### XenaDS.alert()

The alert method takes 1 required parameter and 2 optional (we strongly recommend to use all parameters!)

| Parameter    | Description                                                             |
| ------------ | ----------------------------------------------------------------------- |
| `message`    | The message you want to display                                         |
| `type`       | Type of the alert. This defaults to `primary`                           |
| `header`     | A header that will be shown as a title in the modal that opens          |
| `buttonText` | The close buttons text, ie. 'Close'. This defaults to a localized text. |

#### XenaDS.confirm()

The confirm method takes 1 required parameter and 3 optional (we strongly recommend to use all parameters!)

| Parameter     | Description                                                                              |
| ------------- | ---------------------------------------------------------------------------------------- |
| `message`     | The message you want to display                                                          |
| `type`        | Type of the alert, defaults to primary                                                   |
| `header`      | A header that will be shown as a title in the modal that opens                           |
| `confirmText` | The confirm buttons text, ie. 'Confirm'. This defaults to a localized confirmation text. |
| `cancelText`  | The cancel buttons text, ie. 'Cancel'. This defaults to localized cancel text.           |

```html preview
<div class="alert-dialog-wrapper">
  <xena-button class="alert">Show Alert in Dialog</xena-button>
  <xena-button class="confirm">Show Confirm in Dialog</xena-button>
</div>

<script>
  const container = document.querySelector('.alert-dialog-wrapper');
  const button1 = container.querySelector('.alert');
  const button2 = container.querySelector('.confirm');
  const header = 'A kitty header';
  const message = 'Cats are nice';

  button1.addEventListener('click', () => {
    XenaDS.alert(message, 'primary', header).then(result => console.log('Alert closed'));
  });

  button2.addEventListener('click', () => {
    XenaDS.confirm(message, 'primary', header).then(result => console.log('Confirm closed: ', result));
  });
</script>
```

## Toast

### Toast Notifications

To display an alert as a toast notification, or "toast", create the alert and call its `toast()` method. This will move the alert out of its position in the DOM and into [the toast stack](#the-toast-stack) where it will be shown. Once dismissed, it will be removed from the DOM completely. To reuse a toast, store a reference to it and call `toast()` again later on.

You should always use the `closable` prop so users can dismiss the notification. It's also common to set a reasonable `duration` when the notification doesn't require acknowledgement.

```html preview
<div class="alert-toast">
  <xena-button>Make a toast</xena-button>

  <xena-alert duration="3000" closable>
    <xena-icon slot="icon" name="fas-info-circle"></xena-icon>
    <strong>This is super informative</strong><br />
    You can tell by how pretty the alert is.
  </xena-alert>
</div>

<script>
  const container = document.querySelector('.alert-toast');
  const button = container.querySelector(`xena-button`);
  const alert = container.querySelector(`xena-alert`);

  button.addEventListener('click', () => {
    alert.toast();
  });
</script>
```

?> We only have one type of toast (no "danger", "warning" and so forth). Also, toasts are only informative and nothing else. Learn more about [when to use toasts](ux-guide/communication.md?id=toasts)

### Creating Toasts Imperatively

For convenience, you can create a utility that emits toast notifications with a function call rather than composing them in your HTML. To do this, generate the alert with JavaScript, append it to the body, and call the `toast()` method as shown in the example below.

```html preview
<div class="alert-toast-wrapper">
  <xena-button>Create Toast</xena-button>
</div>

<script>
  const container = document.querySelector('.alert-toast-wrapper');
  const button = container.querySelector('xena-button');
  let count = 0;

  // Always escape HTML for text arguments!
  function escapeHtml(html) {
    const div = document.createElement('div');
    div.textContent = html;
    return div.innerHTML;
  }

  // Custom function to emit toast notifications
  function notify(message, type = 'primary', icon = 'fas-info-circle', duration = 3000) {
    const alert = Object.assign(document.createElement('xena-alert'), {
      type: type,
      closable: true,
      duration: duration,
      innerHTML: `
        <xena-icon name="${icon}" slot="icon"></xena-icon>
        ${escapeHtml(message)}
      `
    });

    document.body.append(alert);
    return alert.toast();
  }

  button.addEventListener('click', () => {
    notify(`This is custom toast #${++count}`);
  });
</script>
```

### The Toast Stack

The toast stack is a fixed position singleton element created and managed internally by the alert component. It will be added and removed from the DOM as needed when toasts are shown. When more than one toast is visible, they will stack vertically in the toast stack.

By default, the toast stack is positioned at the top-right of the viewport. You can change its position by targeting `.xena-notification-stack` in your stylesheet. To make toasts appear at the top-left of the viewport, for example, use the following styles.

```css
.xena-notification-stack {
  left: 0;
  right: auto;
}
```

?> By design, it is not possible to show toasts in more than one stack simultaneously. Such behavior is confusing and makes for a poor user experience.

[component-metadata:xena-alert]
