# Autocomplete

[component-header:xena-autocomplete]

An autocomplete allows users to search and select data.

This autoselect component is simple to use component that builds upon input and dropdown and allows for searching data from a given datasource. It does not require the data elements to be of a specific structure, but by default expects it to have an `id` and a `label` property. However these defaults can be changed via attributes on the element.

In its most simple form it is simply a matter of setting a datasource that can return data. (This one is will not filter the results)

```html preview
<xena-autocomplete placeholder="Type search" id="ac-1"></xena-autocomplete>
<script>
  const data = [
    { label: 'None' },
    { id: '1', label: 'Lasse' },
    { id: '2', label: 'Michael' },
    { id: '3', label: 'Rasmus' },
    { id: '4', label: 'Stefan' }
  ];
  const ac = document.querySelector('#ac-1');
  ac.setDatasource(search => {
    return Promise.resolve(data);
  });
</script>
```

### No results

When no results are available, a text is shown in the dropdown instead. The text can be customized via the no-result-test attribute.

```html preview
<xena-autocomplete placeholder="Type search"></xena-autocomplete>
```

You can also specify that it should not show the dropdown on initial focus on the element.

```html preview
<xena-autocomplete placeholder="Type search" show-initial-no-result="false"></xena-autocomplete>
```

### Scroll

```html preview
<xena-autocomplete placeholder="Type search" id="ac-1-1" max-dropdown-height="100px"></xena-autocomplete>
<script>
  const data = [];

  for (let i = 1; i <= 100; i++) {
    data.push({
      id: '' + i,
      label: `Entry number ${i} ${
        i % 10 == 0
          ? 'followed by a very long text just to ensure that width of this dropdown is also handled correctly'
          : ''
      }`
    });
  }

  const ac = document.querySelector('#ac-1-1');
  ac.setDatasource(search => {
    return Promise.resolve(data);
  });
</script>
```

### Debounce

In many situations the data needs to be searchable and debounced as requests are going to a server or the like. Debounce can be added as a parameter and the search input string can be used for searching.

```html preview
<xena-autocomplete id="ac-2" debounce="300" placeholder="Type search"></xena-autocomplete>
<script>
  const data = [
    { id: '1', label: 'Lasse' },
    { id: '2', label: 'Michael' },
    { id: '3', label: 'Rasmus' },
    { id: '4', label: 'Stefan' }
  ];
  const ac = document.querySelector('#ac-2');

  ac.setDatasource(search => {
    const result = [];
    for (let i = 0; i < data.length; i++) {
      if (search.trim() === '' || data[i].label.toLowerCase().substr(0, search.length) === search.toLowerCase()) {
        result.push(data[i]);
      }
    }

    return new Promise((resolve, reject) => {
      setTimeout(() => resolve(result), 100);
    });
  });

  ac.addEventListener('valueChange', event => {
    const selectedItem = event.detail;
    console.log(selectedItem ? selectedItem.id : null);
  });
</script>
```

### Custom Value Rendering

You may also need to customize the way the value is rendered in the input field or the list. That can be done by specifying `valueLabelProperty` and `listLabelProperty`. This example also shows how to define the properties to use on the objects in use.

```html preview
<xena-autocomplete
  id="ac-3"
  placeholder="Type search"
  id-property="Id"
  list-label-property="Description"
></xena-autocomplete>
<script>
  const data = [
    { Id: '1', Description: 'Lasse is Cool' },
    { Id: '2', Description: 'Michael is Tall' },
    { Id: '3', Description: 'Rasmus is Experienced' },
    { Id: '4', Description: 'Stefan is Awesome' },
    {
      Id: '5',
      Description:
        'Majbritt is the most awesome team leader in the whole wide world - and then some. (READ: Dont mess with her or you will have your glasses turned into an extra set of corneas)'
    }
  ];
  const ac = document.querySelector('#ac-3');

  ac.value = data[0];
  ac.valueLabelProperty = value => (value == null ? '' : `${value.Description}@codezoo`);
  ac.setDatasource(search => {
    const result = [];
    for (let i = 0; i < data.length; i++) {
      if (search.trim() === '' || data[i].Description.toLowerCase().substr(0, search.length) === search.toLowerCase()) {
        result.push(data[i]);
      }
    }

    return Promise.resolve(result);
  });

  ac.addEventListener('valueChange', event => {
    const selectedItem = event.detail;
    console.log(selectedItem ? selectedItem.Id : null);
  });
</script>
```

### Sizes

Use the `size` prop to change an autocomplete's size.

```html preview
<xena-autocomplete size="small" style="margin-bottom:20px"></xena-autocomplete>
<xena-autocomplete size="medium" style="margin-bottom:20px"></xena-autocomplete>
<xena-autocomplete size="large"></xena-autocomplete>
```

### Shapes

Use the `shape` prop to change an autocomplete's shape.

```html preview
<xena-autocomplete shape="round" style="margin-bottom:20px"></xena-autocomplete>
<xena-autocomplete style="margin-bottom:20px"></xena-autocomplete>
<xena-autocomplete shape="edged"></xena-autocomplete>
```

### Prefix and Suffix

Use the `prefix` and `suffix` slots to add icons and text.

```html preview
<xena-autocomplete class="ac-6" style="margin-bottom:20px">
  <span slot="prefix">Search</span>
</xena-autocomplete>

<xena-autocomplete class="ac-6" style="margin-bottom:20px">
  <xena-icon slot="suffix" name="fas-search"></xena-icon>
</xena-autocomplete>

<xena-autocomplete class="ac-6" list-icon-property="icon">
  <span slot="prefix">Find a friend</span>
  <xena-icon slot="suffix" name="fas-search"></xena-icon>
</xena-autocomplete>

<script>
  const data = [
    { id: '1', label: 'Lasse', icon: 'fas-magic' },
    { id: '2', label: 'Michael', icon: 'fas-blind' },
    { id: '3', label: 'Rasmus', icon: 'fas-bed' },
    { id: '4', label: 'Stefan', icon: 'fas-ambulance' }
  ];
  const acs = document.querySelectorAll('.ac-6');
  acs.forEach(ac => {
    ac.setDatasource(search => {
      return Promise.resolve(data);
    });
  });
</script>
```

### Disabled input

At other times you need the element to be disabled which is also supported.

```html preview
<xena-autocomplete disabled></xena-autocomplete>
```

### Labels

Use the `label` attribute to give the select an accessible label. For labels that contain HTML, use the `label` slot instead.

```html preview
<xena-autocomplete id="ac-4" label="Search for developers">
  <div slot="help-text" Wooho></div>
</xena-autocomplete>
<script>
  const data = [
    { id: '1', label: 'Lasse' },
    { id: '2', label: 'Michael' },
    { id: '3', label: 'Rasmus' },
    { id: '4', label: 'Stefan' }
  ];
  const ac = document.querySelector('#ac-4');
  ac.setDatasource(search => {
    return Promise.resolve(data);
  });
</script>
```

### Labels and required

When using the `required` prop with the `label` attribute an asterisk will show that the input is required

```html preview
<xena-autocomplete label="I'm required" required></xena-autocomplete>
```

### Help Text

Add descriptive help text to a select with the `help-text` attribute. For help texts that contain HTML, use the `help-text` slot instead.

```html preview
<xena-autocomplete id="ac-5" label="Pick one" help-text="But remember: the whole team will get it done"></xena-select>
<script>
  const data = [
    { id: '1', label: 'Lasse' },
    { id: '2', label: 'Michael' },
    { id: '3', label: 'Rasmus' },
    { id: '4', label: 'Stefan' }
  ];
  const ac = document.querySelector('#ac-5');
  ac.setDatasource(search => {
    return Promise.resolve(data);
  });
</script>
```

### Error Text

Add descriptive error text to an autocomplete with the `error-text`-attribute. For error texts that contain HTML, use the `error-text`-slot instead. When an error text is supplied the help text will be hidden.

```html preview
<div id="error-example">
  <xena-autocomplete
    label="Enter text"
    help-text="When error is switch off, this help text is shown."
    error-text="The entered value is invalid"
  ></xena-autocomplete>
  <xena-switch class="mt-4" checked>Error enabled</xena-switch>
</div>

<script>
  let example = document.getElementById('error-example');
  let input = example.querySelector('xena-autocomplete');
  let switchEl = example.querySelector('xena-switch');

  switchEl.addEventListener('checkedChange', evt => {
    console.log(evt, input.errorText);
    const checked = evt.detail;
    input.errorText = checked ? 'The entered value is still invalid' : null;
  });
</script>
```

If you are using `error-text`-slot then it will look like this

```html preview
<xena-autocomplete label="Enter text">
  <span slot="error-text">The entered value is invalid</span>
</xena-autocomplete>
```

[component-metadata:xena-autocomplete]
