import { Component, Event, h, Host, Method, Prop, Watch, Element, State } from '@stencil/core';
import { debounce } from '../../utils/events';
import { hasSlot } from '../../utils/slot';
/**
 * @since 1.0
 * @status stable
 *
 * @slot prefix - Used to prepend an icon or text element to the autocompletes input field.
 * @slot suffix - Used to append an icon or text element to the autocompletes input field.
 *
 * @part prefix - The prefix container.
 * @part suffix - The suffix container.
 */
export class Autocomplete {
  constructor() {
    this.focused = false;
    this._text = '';
    this._entries = [];
    /**
     * Set the amount of time, in milliseconds, to wait before initiating search after each keystroke.
     */
    this.debounce = 0;
    /**
     * If `true`, the user cannot interact with the input.
     */
    this.disabled = false;
    /**
     * The text in dropdown when no results are found.
     */
    this.noResultText = 'No results';
    /**
     * The property name for the id of the item.
     */
    this.idProperty = 'id';
    /** The autocomplete's label. */
    this.label = '';
    /** The input's help text.  */
    this.helpText = '';
    /** The input's error text. Alternatively, you can use the error-text slot. */
    this.errorText = '';
    /**
     * The property name for the label of the item shown in the dropdown list.
     */
    this.listLabelProperty = 'label';
    /** Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods. */
    this.open = false;
    /**
     * If `true`, the user must fill in a value before submitting a form.
     */
    this.required = false;
    /**
     * Defines the shape of the input field.
     */
    this.shape = 'default';
    /**
     * Defines the size of the input field.
     */
    this.size = 'medium';
    /**
     * The selected value of the component.
     */
    this.value = null;
    /**
     * The property name for the label of the item shown in the input field.
     */
    this.valueLabelProperty = 'label';
    /**
     * Whether the dropdown with results should be shown even when there are no results on the components initial focus.
     */
    this.showInitialNoResult = true;
  }
  debounceChanged() {
    this._debounceInput = debounce(ev => {
      ev.stopPropagation();
      this._text = this.input.value;
      if (this.focused && (this.value == null || this._text != this.resolveValueLabel(this.value))) {
        this.search().then(() => this.dropdown.show());
      }
    }, this.debounce);
  }
  /**
   * Sets the datasource to retrieve data from. The datasource is a lambda function which can expect a search query as an argument.
   * In return it must provide a Promise that can deliver an array of data entries.
   * @param datasource
   */
  async setDatasource(datasource) {
    this._datasource = datasource;
  }
  /**
   * Sets focus on the input field.
   */
  async setFocus() {
    this.input.setFocus();
  }
  /**
   * Retrieves the native input element.
   */
  async getInputElement() {
    return this.input.getInputElement();
  }
  /**
   * Closes the autocomplete popup if open.
   */
  async hide() {
    this.dropdown.hide();
  }
  /**
   * Opens the autocomplete popup if closed.
   */
  async show() {
    return this.search().then(() => this.dropdown.show());
  }
  componentWillLoad() {
    this.valueChanged();
    this.handleSlotChange();
  }
  componentDidRender() {
    if (this.list && typeof this.list.getMenuElement === 'function') {
      this.list.getMenuElement().then(menu => menu.style.setProperty('max-height', this.maxDropdownHeight));
    }
  }
  connectedCallback() {
    this.debounceChanged();
    if ('MutationObserver' in window) {
      this.mutationObserver = new MutationObserver(() => this.handleSlotChange());
      this.mutationObserver.observe(this.host, { attributes: false, childList: true, subtree: false });
    }
  }
  disconnectedCallback() {
    var _a;
    (_a = this.mutationObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
  }
  valueChanged() {
    this._text = this.resolveValueLabel(this.value);
    if (this.input) {
      this.input.value = this._text;
    }
  }
  handleSlotChange() {
    this.hasPrefixSlot = hasSlot(this.host, 'prefix');
    this.hasSuffixSlot = hasSlot(this.host, 'suffix');
    this.hasErrorTextSlot = hasSlot(this.host, 'error-text');
  }
  search() {
    if (!this._datasource)
      return Promise.resolve();
    return this._datasource(this._text).then(result => {
      this._entries = result;
      this.list.entries = result;
      this.updateDefaultActiveIndex();
      if (this._entries == null || this._entries.length == 0) {
        return;
      }
      setTimeout(() => {
        try {
          this.dropdown.scheduleUpdate();
        }
        catch (error) {
          /** IGNORE ERROR */
        }
      }, 10);
    });
  }
  handleChange(ev) {
    this._debounceInput(ev);
  }
  handleSelection(ev) {
    ev.stopPropagation();
    this.dropdown.hide();
    const item = ev.detail.item;
    const id = item.value;
    this.value = null;
    if (id != null) {
      this._entries.forEach(e => {
        if (this.resolveId(e) == id) {
          this.value = e;
        }
      });
    }
    this.valueChange.emit(this.value);
  }
  handleFocus() {
    this.focused = true;
    if (this.value == null && !this.open) {
      this.search().then(() => {
        if ((this._entries != null && this._entries.length > 0) || this.showInitialNoResult) {
          this.dropdown.show();
        }
      });
    }
  }
  handleBlur() {
    this.focused = false;
    if (this.input) {
      this.input.value = this._text;
    }
    this.checkIfCleared();
  }
  handleDropdownShow() {
    this.updateDefaultActiveIndex();
    this.open = true;
  }
  updateDefaultActiveIndex() {
    const activeIndex = this._entries.length > 0 ? 0 : undefined;
    this.list.defaultActiveIndex = activeIndex;
  }
  handleDropdownAfterHide() {
    this.open = false;
    this.checkIfCleared();
    this.valueChanged();
  }
  checkIfCleared() {
    if ((!this._text || this._text.trim() === '') && this.value != null && !this.open) {
      this.value = null;
      this.valueChange.emit(this.value);
    }
  }
  resolveId(entry) {
    return entry[this.idProperty];
  }
  resolveValueLabel(entry) {
    if (entry == null)
      return '';
    if (typeof this.valueLabelProperty === 'function') {
      return this.valueLabelProperty(entry);
    }
    else {
      return entry[this.valueLabelProperty];
    }
  }
  handleKeyDown(event) {
    // Select active element when tabbing out of autocomplete
    if (['Tab', 'Enter'].includes(event.key)) {
      this.list.getMenuElement().then(el => el.selectActiveMenuElement());
      if (event.key === 'Tab' && this._entries.length === 0) {
        this.hide();
      }
    }
  }
  handleValueChanged(event) {
    event.stopPropagation();
  }
  render() {
    return (h(Host, null,
      h("xena-dropdown", { ref: el => (this.dropdown = el), hoist: true, triggerClickAction: 'none', noToggleKey: true, onXnShow: _ev => this.handleDropdownShow(), onXnAfterHide: _ev => this.handleDropdownAfterHide() },
        h("xena-input", { label: this.label, helpText: this.helpText, errorText: this.errorText, ref: el => (this.input = el), disabled: this.disabled, slot: "trigger", placeholder: this.placeholder, value: this._text, onFocus: _ev => this.handleFocus(), onBlur: _ev => this.handleBlur(), onXnInput: _ev => this.handleChange(_ev), onValueChange: _ev => this.handleValueChanged(_ev), shape: this.shape, size: this.size, onKeyDown: _ev => this.handleKeyDown(_ev), required: this.required },
          this.hasPrefixSlot ? (h("span", { slot: "prefix", part: "prefix" },
            h("slot", { name: "prefix" }))) : (''),
          this.hasSuffixSlot ? (h("span", { slot: "suffix", part: "suffix" },
            h("slot", { name: "suffix" }))) : (''),
          this.hasErrorTextSlot ? (h("span", { slot: "error-text", part: "error-text" },
            h("slot", { name: "error-text" }))) : ('')),
        h("xena-autocomplete-list", { ref: el => (this.list = el), noResultText: this.noResultText, entries: this._entries, onXnSelect: ev => this.handleSelection(ev), idProperty: this.idProperty, labelProperty: this.listLabelProperty, iconProperty: this.listIconProperty }))));
  }
  static get is() { return "xena-autocomplete"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["autocomplete.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["autocomplete.css"]
  }; }
  static get properties() { return {
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set the amount of time, in milliseconds, to wait before initiating search after each keystroke."
      },
      "attribute": "debounce",
      "reflect": false,
      "defaultValue": "0"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the user cannot interact with the input."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "maxDropdownHeight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Specified the max height of the dropdown."
      },
      "attribute": "max-dropdown-height",
      "reflect": false
    },
    "noResultText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The text in dropdown when no results are found."
      },
      "attribute": "no-result-text",
      "reflect": false,
      "defaultValue": "'No results'"
    },
    "idProperty": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The property name for the id of the item."
      },
      "attribute": "id-property",
      "reflect": false,
      "defaultValue": "'id'"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The autocomplete's label."
      },
      "attribute": "label",
      "reflect": false,
      "defaultValue": "''"
    },
    "helpText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The input's help text."
      },
      "attribute": "help-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "errorText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The input's error text. Alternatively, you can use the error-text slot."
      },
      "attribute": "error-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "listLabelProperty": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string | ((e: any) => string)",
        "resolved": "((e: any) => string) | string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The property name for the label of the item shown in the dropdown list."
      },
      "attribute": "list-label-property",
      "reflect": false,
      "defaultValue": "'label'"
    },
    "listIconProperty": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string | ((e: any) => string)",
        "resolved": "((e: any) => string) | string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The property name for the icon of the item shown in the dropdown list. The name of the icon must be for an icon in the default icon set. (See xena-icon)"
      },
      "attribute": "list-icon-property",
      "reflect": false
    },
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the user must fill in a value before submitting a form."
      },
      "attribute": "required",
      "reflect": false,
      "defaultValue": "false"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The text to show in the input field when user is not searching."
      },
      "attribute": "placeholder",
      "reflect": false
    },
    "shape": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ShapeTypes",
        "resolved": "\"default\" | \"edged\" | \"round\"",
        "references": {
          "ShapeTypes": {
            "location": "import",
            "path": "../../utils/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the shape of the input field."
      },
      "attribute": "shape",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "SizeTypes",
        "resolved": "\"large\" | \"medium\" | \"small\"",
        "references": {
          "SizeTypes": {
            "location": "import",
            "path": "../../utils/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the size of the input field."
      },
      "attribute": "size",
      "reflect": false,
      "defaultValue": "'medium'"
    },
    "value": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The selected value of the component."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "null"
    },
    "valueLabelProperty": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string | ((e: any) => string)",
        "resolved": "((e: any) => string) | string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The property name for the label of the item shown in the input field."
      },
      "attribute": "value-label-property",
      "reflect": false,
      "defaultValue": "'label'"
    },
    "showInitialNoResult": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether the dropdown with results should be shown even when there are no results on the components initial focus."
      },
      "attribute": "show-initial-no-result",
      "reflect": false,
      "defaultValue": "true"
    }
  }; }
  static get states() { return {
    "hasPrefixSlot": {},
    "hasSuffixSlot": {},
    "hasErrorTextSlot": {}
  }; }
  static get events() { return [{
      "method": "valueChange",
      "name": "valueChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when value has been changed - either by selecting an item from the list or by deleting the selected value."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "setDatasource": {
      "complexType": {
        "signature": "(datasource: (searchString: string) => Promise<any[]>) => Promise<void>",
        "parameters": [{
            "tags": [{
                "name": "param",
                "text": "datasource"
              }],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets the datasource to retrieve data from. The datasource is a lambda function which can expect a search query as an argument.\nIn return it must provide a Promise that can deliver an array of data entries.",
        "tags": [{
            "name": "param",
            "text": "datasource"
          }]
      }
    },
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets focus on the input field.",
        "tags": []
      }
    },
    "getInputElement": {
      "complexType": {
        "signature": "() => Promise<HTMLInputElement>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLInputElement>"
      },
      "docs": {
        "text": "Retrieves the native input element.",
        "tags": []
      }
    },
    "hide": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the autocomplete popup if open.",
        "tags": []
      }
    },
    "show": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Opens the autocomplete popup if closed.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "debounce",
      "methodName": "debounceChanged"
    }, {
      "propName": "value",
      "methodName": "valueChanged"
    }]; }
}
