import { Component, Prop, h, State, Element, Event, Watch } from '@stencil/core';
import { Colors } from '../../utils/colors';
/**
 * @since 1.8
 * @status experimental
 *
 * @part base - The component's base wrapper.
 */
export class ColorPicker {
  constructor() {
    this.hue = 0;
    this.saturation = 0;
    this.lightness = 0;
    this.showCustomColorPicker = false;
    this.isDragging = false;
    this.customToggle = true;
    /** The components value. The value is set by selecting a color with the color picker. */
    this.value = '';
    /** Swatches are predefined colors shown at the top of the color picker. Swatches must be written as hex values with `#` as prefix like this: `#517770`. */
    this.swatches = '';
    /** The components type. */
    this.type = 'default';
    this.stayInHundreds = (value) => {
      if (value < 0) {
        return 0;
      }
      if (value > 100) {
        return 100;
      }
      return value;
    };
  }
  valueChangedHandler(value) {
    this.value = value;
    this.valueChange.emit(this.value);
  }
  componentWillLoad() {
    this.checkForSwatches();
    this.updateColorHexHsl();
    this.checkForType();
  }
  componentDidLoad() {
    this.dropdown = this.element.shadowRoot.querySelector('xena-dropdown');
  }
  checkForType() {
    if (this.type === 'default') {
      this.showCustomColorPicker = true;
    }
  }
  parseSwatches(swatches) {
    if (!swatches.trim()) {
      return [];
    }
    else {
      return swatches
        .split(';')
        .map(s => s.trim())
        .filter(s => s !== '');
    }
  }
  toggleVisibility() {
    this.showCustomColorPicker = !this.showCustomColorPicker;
    this.customToggle = !this.customToggle;
    this.updateColorHexHsl();
  }
  checkForSwatches() {
    const swatchesArray = this.parseSwatches(this.swatches);
    if (swatchesArray.length === 0) {
      this.showCustomColorPicker = !this.showCustomColorPicker;
    }
  }
  handleSwatchClick(hexValue) {
    this.value = hexValue;
    this.updateColorHexHsl();
    this.valueChange.emit(this.value);
  }
  handleColorInput(event) {
    if (this.isDragging)
      return;
    const inputElement = event.target;
    let value = inputElement.value;
    if (!value.startsWith('#')) {
      value = '#' + value;
    }
    this.value = value;
    this.updateColorHexHsl();
  }
  handleInputEnter(event) {
    if (event.key === 'Enter') {
      this.dropdown.hide();
      event.stopPropagation();
    }
  }
  hueChangeHandler(event) {
    const value = event.detail;
    this.hue = value.x;
    this.updateColorHslHex();
    event.stopPropagation();
  }
  toneChangeHandler(event) {
    const value = event.detail;
    this.saturation = value.x;
    this.lightness = value.y * (100 - value.x * 0.5) * 0.01;
    this.updateColorHslHex();
    event.stopPropagation();
  }
  updateColorHslHex() {
    const selectedHue = this.hue;
    const saturationValue = this.saturation;
    const lightnessValue = this.lightness;
    const { r, g, b } = Colors.hslToRgb(selectedHue, saturationValue, lightnessValue);
    const hexColor = Colors.rgbToHex(r, g, b);
    this.value = hexColor;
  }
  updateColorHexHsl() {
    const hexValue = this.value;
    const { r, g, b } = Colors.hexToRgb(hexValue);
    const { h, s, l } = Colors.rgbToHsl(r, g, b);
    this.hue = h;
    this.saturation = s;
    this.lightness = l;
  }
  render() {
    const swatchesArray = this.parseSwatches(this.swatches);
    const placeholder = '#517770';
    const getHue = { x: this.hue, y: 0 };
    const getTone = {
      x: this.saturation,
      y: this.stayInHundreds(this.lightness / (0.01 * (100 - 0.5 * this.saturation)))
    };
    return (h("xena-dropdown", null,
      h("div", { id: "colorInput", slot: "trigger", class: "color", style: { backgroundColor: this.value } }),
      h("div", { class: "color-picker", part: "base" },
        swatchesArray.length > 0 && (h("div", { class: "swatches" },
          h("div", { class: "swatches-colors" }, swatchesArray.map(hexValue => (h("div", { class: "color", style: { backgroundColor: hexValue }, onClick: () => this.handleSwatchClick(hexValue) })))),
          this.type === 'expandable' && this.customToggle ? (h("a", { id: "custom-toggle", href: "javascript:void(0)", role: "button", onClick: () => this.toggleVisibility() },
            h("xena-icon", { name: "fas-angle-down" }))) : null)),
        this.showCustomColorPicker === true ? (h("div", { id: "custom-color" },
          h("div", { id: "color" },
            h("xena-slider", { onValueChange: e => this.toneChangeHandler(e), type: "both", min: 0, max: 100, value: getTone, style: {
                background: `
                      linear-gradient(rgba(255,255,255,0), rgba(0,0,0,1)), 
                      linear-gradient(to right, hsl(${this.hue} 100% 50% / 0), hsl(${this.hue} 100% 50% / 1)`
              }, width: "100%" })),
          h("div", { class: "color-options" },
            h("div", { id: "hue" },
              h("xena-slider", { onValueChange: e => this.hueChangeHandler(e), type: "horizontal", min: 0, max: 360, value: getHue, width: "100%" })),
            h("div", { class: "input-selected" },
              h("xena-input", { value: this.value, onInput: event => this.handleColorInput(event), onKeyDown: event => this.handleInputEnter(event), placeholder: placeholder },
                h("span", { slot: "prefix" }, "Hex")),
              h("div", { class: "color cursor-default", style: { backgroundColor: this.value } })),
            this.type === 'expandable' && !this.customToggle ? (h("a", { id: "custom-toggle", href: "javascript:void(0)", role: "button", onClick: () => this.toggleVisibility() },
              h("xena-icon", { name: "fas-angle-up" }))) : null))) : null)));
  }
  static get is() { return "xena-color-picker"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["color-picker.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["color-picker.css"]
  }; }
  static get properties() { return {
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The components value. The value is set by selecting a color with the color picker."
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "''"
    },
    "swatches": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Swatches are predefined colors shown at the top of the color picker. Swatches must be written as hex values with `#` as prefix like this: `#517770`."
      },
      "attribute": "swatches",
      "reflect": true,
      "defaultValue": "''"
    },
    "type": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "'default' | 'expandable' | 'swatches'",
        "resolved": "\"default\" | \"expandable\" | \"swatches\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The components type."
      },
      "attribute": "type",
      "reflect": true,
      "defaultValue": "'default'"
    }
  }; }
  static get states() { return {
    "hue": {},
    "saturation": {},
    "lightness": {},
    "showCustomColorPicker": {}
  }; }
  static get events() { return [{
      "method": "valueChange",
      "name": "valueChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "A hex value is emitted whenever the color is changed by swatches, sliders, or input."
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get elementRef() { return "element"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "valueChangedHandler"
    }]; }
}
