import { Component, Element, Event, Method, Prop, Watch, h } from '@stencil/core';
import { scrollIntoView } from '../../utils/scroll';
import Popover from '../../utils/popover';
let id = 0;
/**
 * @since 1.0
 * @status stable
 *
 * @slot trigger - The dropdown's trigger, usually a `<xena-button>` element.
 * @slot - The dropdown's content.
 *
 * @part base - The component's base wrapper.
 * @part trigger - The container that wraps the trigger.
 * @part panel - The panel that gets shown when the dropdown is open.
 */
export class Dropdown {
  constructor() {
    this.componentId = `dropdown-${++id}`;
    this.isShowing = false;
    this.triggerIsEditable = false;
    /** Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods. */
    this.open = false;
    /**
     * The preferred placement of the dropdown panel. Note that the actual placement may vary as needed to keep the panel
     * inside of the viewport.
     */
    this.placement = 'bottom-start';
    /** Determines whether the dropdown should hide when a menu item is selected. */
    this.closeOnSelect = true;
    /** The distance in pixels from which to offset the panel away from its trigger. */
    this.distance = 2;
    /** The distance in pixels from which to offset the panel along its trigger. */
    this.skidding = 0;
    /**
     * Enable this option to prevent the panel from being clipped when the component is placed inside a container with
     * `overflow: auto|scroll`.
     */
    this.hoist = false;
    /**
     * Disables toggle keys for dropdown. Keys for toggling will only close the dropdown, not open it.
     */
    this.noToggleKey = false;
    /**
     * Action taken against the dropdown when trigger is clicked.
     */
    this.triggerClickAction = 'toggle';
    /**
     * Show arrow on the dropdown pointing to the trigger.
     */
    this.arrow = false;
  }
  handleOpenChange() {
    this.open ? this.show() : this.hide();
  }
  getDistance() {
    return this.arrow ? this.distance + 10 : this.distance;
  }
  handlePopoverOptionsChange() {
    this.popover.setOptions({
      strategy: this.hoist ? 'fixed' : 'absolute',
      placement: this.placement,
      distance: this.getDistance(),
      skidding: this.skidding
    });
  }
  connectedCallback() {
    if (!this.containingElement) {
      this.containingElement = this.host;
    }
    this.handleDocumentKeyDown = this.handleDocumentKeyDown.bind(this);
    this.handleDocumentMouseDown = this.handleDocumentMouseDown.bind(this);
    this.handleMenuItemActivate = this.handleMenuItemActivate.bind(this);
    this.handlePanelSelect = this.handlePanelSelect.bind(this);
    this.handleTriggerClick = this.handleTriggerClick.bind(this);
    this.handleTriggerKeyDown = this.handleTriggerKeyDown.bind(this);
    this.handleTriggerKeyUp = this.handleTriggerKeyUp.bind(this);
  }
  componentDidLoad() {
    this.popover = new Popover(this.trigger, this.positioner, {
      strategy: this.hoist ? 'fixed' : 'absolute',
      placement: this.placement,
      distance: this.getDistance(),
      skidding: this.skidding,
      transitionElement: this.panel,
      onAfterHide: () => this.xnAfterHide.emit(),
      onAfterShow: () => this.xnAfterShow.emit(),
      onTransitionEnd: () => {
        if (!this.open) {
          this.panel.scrollTop = 0;
        }
      }
    });
    // Show on init if open
    if (this.open) {
      this.show();
    }
    const slot = this.trigger.querySelector('slot');
    if (slot.assignedElements) {
      const trigger = slot.assignedElements({ flatten: true })[0];
      if (typeof trigger.disabled === 'boolean' &&
        typeof trigger.readonly === 'boolean' &&
        typeof trigger.type === 'string' &&
        trigger.readonly === false &&
        trigger.disabled === false) {
        this.triggerIsEditable = true;
      }
      else {
        this.triggerIsEditable = false;
      }
    }
  }
  disconnectedCallback() {
    var _a;
    this.hide();
    (_a = this.popover) === null || _a === void 0 ? void 0 : _a.destroy();
  }
  /** Shows the dropdown panel */
  async show() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (this.isShowing) {
      return;
    }
    const xnShow = this.xnShow.emit();
    if (xnShow.defaultPrevented) {
      this.open = false;
      return;
    }
    this.panel.addEventListener('xnActivate', this.handleMenuItemActivate);
    this.panel.addEventListener('xnSelect', this.handlePanelSelect);
    document.addEventListener('keydown', this.handleDocumentKeyDown);
    document.addEventListener('mousedown', this.handleDocumentMouseDown);
    this.isShowing = true;
    this.open = true;
    this.popover.show();
  }
  /** Hides the dropdown panel */
  async hide() {
    // Prevent subsequent calls to the method, whether manually or triggered by the `open` watcher
    if (!this.isShowing) {
      return;
    }
    const slHide = this.xnHide.emit();
    if (slHide.defaultPrevented) {
      this.open = true;
      return;
    }
    this.panel.removeEventListener('xnActivate', this.handleMenuItemActivate);
    this.panel.removeEventListener('xnSelect', this.handlePanelSelect);
    document.addEventListener('keydown', this.handleDocumentKeyDown);
    document.removeEventListener('mousedown', this.handleDocumentMouseDown);
    this.isShowing = false;
    this.open = false;
    this.popover.hide();
  }
  /**
   * This method can be called to force the dropdown to reposition itself. In most cases it should do so automatically,
   * but there may be some edge cases where changes are not detected.
   */
  async scheduleUpdate() {
    this.popover.update();
  }
  focusOnTrigger() {
    const trigger = this.resolveContainedTrigger();
    if (trigger) {
      if (typeof trigger.setFocus === 'function') {
        trigger.setFocus();
      }
      else if (typeof trigger.focus === 'function') {
        trigger.focus();
      }
    }
  }
  resolveContainedTrigger() {
    const slot = this.trigger.querySelector('slot');
    const trigger = slot.assignedElements({ flatten: true })[0];
    return trigger;
  }
  getMenu() {
    const list = this.panel.querySelector('slot').assignedElements({ flatten: true });
    for (let i = 0; i < list.length; i++) {
      const el = list[i];
      if (el.tagName.toLowerCase() === 'xena-menu') {
        return el;
      }
      else {
        for (let e = 0; e < el.children.length; e++) {
          const child = el.children[e];
          if (child.tagName.toLowerCase() === 'xena-menu') {
            return child;
          }
        }
        if (!el.shadowRoot)
          return;
        for (let e = 0; e < el.shadowRoot.children.length; e++) {
          const child = el.shadowRoot.children[e];
          if (child.tagName.toLowerCase() === 'xena-menu') {
            return child;
          }
        }
      }
    }
  }
  handleDocumentKeyDown(event) {
    // Close when escape is pressed
    if (event.key === 'Escape' && this.open) {
      this.hide();
      this.focusOnTrigger();
      return;
    }
    // Handle tabbing & Horizontal Arrows
    if (['ArrowLeft', 'ArrowRight', 'Tab'].includes(event.key)) {
      setTimeout(() => {
        var _a;
        // Tabbing outside of the containing element closes the panel
        //
        // If the dropdown is used within a shadow DOM, we need to obtain the activeElement within that shadowRoot,
        // otherwise `document.activeElement` will only return the name of the parent shadow DOM element.
        const activeElement = this.containingElement.getRootNode() instanceof ShadowRoot
          ? (_a = document.activeElement.shadowRoot) === null || _a === void 0 ? void 0 : _a.activeElement
          : document.activeElement;
        if ((activeElement === null || activeElement === void 0 ? void 0 : activeElement.closest(this.containingElement.tagName.toLowerCase())) !== this.containingElement) {
          this.hide();
          return;
        }
      });
    }
  }
  handleDocumentMouseDown(event) {
    // Close when clicking outside of the containing element
    const path = event.composedPath();
    if (!path.includes(this.containingElement)) {
      this.hide();
      return;
    }
  }
  handleMenuItemActivate(event) {
    if (event.target.tagName !== 'XENA-MENU-ITEM')
      return;
    const item = event.target;
    scrollIntoView(item, this.panel);
  }
  handlePanelSelect(event) {
    const target = event.target;
    // Hide the dropdown when a menu item is selected
    if (this.closeOnSelect && target.tagName.toLowerCase() === 'xena-menu') {
      this.hide();
      this.focusOnTrigger();
    }
  }
  handleTriggerClick() {
    if (this.resolveContainedTrigger().disabled || this.triggerClickAction === 'none') {
      return;
    }
    if (this.triggerClickAction === 'toggle') {
      this.open ? this.hide() : this.show();
    }
    if (this.triggerClickAction === 'open') {
      this.show();
    }
  }
  handleTriggerKeyDown(event) {
    const menu = this.getMenu();
    // Close when escape or tab is pressed
    if (event.key === 'Escape') {
      this.hide();
      this.focusOnTrigger();
      return;
    }
    // When spacebar/enter is pressed, show the panel but don't focus on the menu. This let's the user press the same
    // key again to hide the menu in case they don't want to make a selection.
    const toggleKeys = !this.triggerIsEditable ? [' ', 'Enter'] : ['Enter'];
    if (toggleKeys.includes(event.key)) {
      event.preventDefault();
      if (!this.noToggleKey || this.isShowing) {
        this.open ? this.hide() : this.show();
      }
      return;
    }
    // When up/down is pressed, we make the assumption that the user is familiar with the menu and plans to make a
    // selection. Rather than toggle the panel, we focus on the menu (if one exists) and activate the first item for
    // faster navigation. If noToggleKey is true, then only enter if the menu is already open.
    if (['ArrowDown', 'ArrowUp'].includes(event.key)) {
      event.preventDefault();
      if (!this.noToggleKey || this.open) {
        // Show the menu if it's not already open
        if (!this.open) {
          this.show();
        }
        // Focus on the menu, if one exists
        if (menu) {
          menu.setFocus();
          return;
        }
      }
      else {
        return;
      }
    }
    // Other keys bring focus to the menu and initiate type-to-select behavior
    const ignoredKeys = ['Tab', 'Shift', 'Meta', 'Ctrl', 'Alt', ' '];
    if (this.open && menu && !ignoredKeys.includes(event.key) && !this.triggerIsEditable) {
      menu.setFocus();
      menu.typeToSelect(event.key);
      return;
    }
  }
  handleTriggerKeyUp(event) {
    // Prevent space from triggering a click event in Firefox
    if (event.key === ' ' && !this.triggerIsEditable) {
      event.preventDefault();
    }
  }
  render() {
    return (h("div", { part: "base", id: this.componentId, class: {
        dropdown: true,
        dropdown__open: this.open
      }, "aria-expanded": this.open, "aria-haspopup": "true" },
      h("span", { part: "trigger", class: "dropdown__trigger", ref: el => (this.trigger = el), onClick: this.handleTriggerClick, onKeyDown: this.handleTriggerKeyDown, onKeyUp: this.handleTriggerKeyUp },
        h("slot", { name: "trigger" })),
      h("div", { ref: el => (this.positioner = el), class: "dropdown__positioner" },
        this.arrow && h("div", { class: "dropdown__arrow" }),
        h("div", { ref: el => (this.panel = el), part: "panel", class: "dropdown__panel", role: "menu", "aria-hidden": !this.open, "aria-labelledby": this.componentId },
          h("slot", null)))));
  }
  static get is() { return "xena-dropdown"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["dropdown.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["dropdown.css"]
  }; }
  static get properties() { return {
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "placement": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "| 'top'\n    | 'top-start'\n    | 'top-end'\n    | 'bottom'\n    | 'bottom-start'\n    | 'bottom-end'\n    | 'right'\n    | 'right-start'\n    | 'right-end'\n    | 'left'\n    | 'left-start'\n    | 'left-end'",
        "resolved": "\"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The preferred placement of the dropdown panel. Note that the actual placement may vary as needed to keep the panel\ninside of the viewport."
      },
      "attribute": "placement",
      "reflect": false,
      "defaultValue": "'bottom-start'"
    },
    "closeOnSelect": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Determines whether the dropdown should hide when a menu item is selected."
      },
      "attribute": "close-on-select",
      "reflect": false,
      "defaultValue": "true"
    },
    "containingElement": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "HTMLElement",
        "resolved": "HTMLElement",
        "references": {
          "HTMLElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The dropdown will close when the user interacts outside of this element (e.g. clicking)."
      }
    },
    "distance": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The distance in pixels from which to offset the panel away from its trigger."
      },
      "attribute": "distance",
      "reflect": false,
      "defaultValue": "2"
    },
    "skidding": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The distance in pixels from which to offset the panel along its trigger."
      },
      "attribute": "skidding",
      "reflect": false,
      "defaultValue": "0"
    },
    "hoist": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Enable this option to prevent the panel from being clipped when the component is placed inside a container with\n`overflow: auto|scroll`."
      },
      "attribute": "hoist",
      "reflect": false,
      "defaultValue": "false"
    },
    "noToggleKey": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables toggle keys for dropdown. Keys for toggling will only close the dropdown, not open it."
      },
      "attribute": "no-toggle-key",
      "reflect": false,
      "defaultValue": "false"
    },
    "triggerClickAction": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'toggle' | 'open' | 'none'",
        "resolved": "\"none\" | \"open\" | \"toggle\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Action taken against the dropdown when trigger is clicked."
      },
      "attribute": "trigger-click-action",
      "reflect": false,
      "defaultValue": "'toggle'"
    },
    "arrow": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Show arrow on the dropdown pointing to the trigger."
      },
      "attribute": "arrow",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "xnShow",
      "name": "xnShow",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the dropdown opens. Calling `event.preventDefault()` will prevent it from being opened."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnAfterShow",
      "name": "xnAfterShow",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the dropdown opens and all transitions are complete."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnHide",
      "name": "xnHide",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the dropdown closes. Calling `event.preventDefault()` will prevent it from being closed."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "xnAfterHide",
      "name": "xnAfterHide",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the dropdown closes and all transitions are complete."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "show": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Shows the dropdown panel",
        "tags": []
      }
    },
    "hide": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Hides the dropdown panel",
        "tags": []
      }
    },
    "scheduleUpdate": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "This method can be called to force the dropdown to reposition itself. In most cases it should do so automatically,\nbut there may be some edge cases where changes are not detected.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "host"; }
  static get watchers() { return [{
      "propName": "open",
      "methodName": "handleOpenChange"
    }, {
      "propName": "distance",
      "methodName": "handlePopoverOptionsChange"
    }, {
      "propName": "hoist",
      "methodName": "handlePopoverOptionsChange"
    }, {
      "propName": "placement",
      "methodName": "handlePopoverOptionsChange"
    }, {
      "propName": "skidding",
      "methodName": "handlePopoverOptionsChange"
    }]; }
}
