import { Host, Component, h, Prop, Event, Element, Method, State, Watch } from '@stencil/core';
import { Dates } from '../../utils/dates';
/**
 * @since 1.0
 * @status experimental
 *
 * @slot prefix - Used to prepend an icon or text element to the autocompletes input field.
 * @slot suffix - Used to append an icon or text element to the autocompletes input field.
 *
 * @part input - The component's input field
 * @part calendar - The calendar popup.
 * @part prefix - The prefix container.
 * @part suffix - The suffix container.
 */
export class FiscalDatepicker {
  constructor() {
    this.isFocused = false;
    this.text = '';
    /**
     * Defines the currently selected date. If not set, the current date date will be used.
     */
    this.date = Dates.dateToFiscalDate(new Date());
    /**
     * Defines the systemDate as a fiscalDate. If not set, the current date will automatically be selected.
     */
    this.systemDate = -1;
    /**
     * If `true`, the user cannot interact with the input.
     */
    this.disabled = false;
    /** The datepicker's label. */
    this.label = '';
    /** The datepicker's help text. */
    this.helpText = '';
    /** Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods. */
    this.open = false;
    /**
     * Defines the shape of the input field.
     */
    this.shape = 'default';
    /**
     * Defines the size of the input field.
     */
    this.size = 'medium';
  }
  /**
   * Retrieves the native input element.
   */
  async getInputElement() {
    return this.textInput.getInputElement();
  }
  /**
   * Set focus on the native input element.
   */
  async setFocus() {
    this.textInput.setFocus();
  }
  /**
   * Closes the calendar popup if open.
   */
  async hide() {
    this.dropdown.hide();
  }
  /**
   * Opens the calendar popup if closed.
   */
  async show() {
    this.dropdown.show();
  }
  handleTextChange() {
    if (this.textInput) {
      this.textInput.value = this.text;
    }
  }
  componentWillLoad() {
    if (this.systemDate < 0) {
      this.systemDate = Dates.dateToFiscalDate(new Date());
    }
    this.text = this.formattedDate;
    this.hasPrefixSlot = !!this.el.querySelector('[slot="prefix"]');
    this.hasSuffixSlot = !!this.el.querySelector('[slot="suffix"]');
  }
  handleDropdownShow() {
    this.open = true;
  }
  handleDropdownAfterHide() {
    this.open = false;
  }
  handleInputChange(_ev) {
    if (!this.isFocused)
      return;
    this.dropdown.show();
    var value = this.textInput.value;
    this.text = value;
    var fdate = Dates.stringToFiscalDate(this.text, this.systemDate);
    if ((!fdate && value !== '') || fdate < 1)
      return;
    this.calendar.date = fdate;
  }
  updateValues(date) {
    this.text = this.formattedDate;
    if (date === this.date)
      return;
    this.date = date;
    if (this.calendar.date != this.date) {
      this.calendar.date = this.date;
    }
    this.text = this.formattedDate;
    this.dateChange.emit(this.date);
  }
  dateSelected(date) {
    this.updateValues(date);
    this.dropdown.hide();
  }
  handleFocus() {
    this.isFocused = true;
    this.dropdown.show();
  }
  handleBlur() {
    this.isFocused = false;
    this.updateValues(this.calendar.date);
  }
  get formattedDate() {
    return Dates.format(Dates.fiscalDateToDate(this.date));
  }
  render() {
    return (h(Host, null,
      h("xena-dropdown", { noToggleKey: true, triggerClickAction: 'none', hoist: true, ref: el => (this.dropdown = el), onXnShow: _ev => this.handleDropdownShow(), onXnAfterHide: _ev => this.handleDropdownAfterHide() },
        h("xena-input", { label: this.label, helpText: this.helpText, disabled: this.disabled, part: "input", slot: "trigger", onFocus: () => this.handleFocus(), onBlur: () => this.handleBlur(), onValueChange: ev => this.handleInputChange(ev), value: this.text, shape: this.shape, size: this.size, ref: el => (this.textInput = el) },
          this.hasPrefixSlot ? (h("span", { slot: "prefix", part: "prefix" },
            h("slot", { name: "prefix" }))) : (''),
          this.hasSuffixSlot ? (h("span", { slot: "suffix", part: "suffix" },
            h("slot", { name: "suffix" }))) : ('')),
        h("xena-fiscal-calendar", { part: "calendar", ref: el => (this.calendar = el), date: this.date, onDateChange: e => this.dateSelected(e.detail) }))));
  }
  static get is() { return "xena-fiscal-datepicker"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["fiscal-datepicker.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["fiscal-datepicker.css"]
  }; }
  static get properties() { return {
    "date": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the currently selected date. If not set, the current date date will be used."
      },
      "attribute": "date",
      "reflect": false,
      "defaultValue": "Dates.dateToFiscalDate(new Date())"
    },
    "systemDate": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the systemDate as a fiscalDate. If not set, the current date will automatically be selected."
      },
      "attribute": "system-date",
      "reflect": false,
      "defaultValue": "-1"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the user cannot interact with the input."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The datepicker's label."
      },
      "attribute": "label",
      "reflect": false,
      "defaultValue": "''"
    },
    "helpText": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The datepicker's help text."
      },
      "attribute": "help-text",
      "reflect": false,
      "defaultValue": "''"
    },
    "open": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods."
      },
      "attribute": "open",
      "reflect": true,
      "defaultValue": "false"
    },
    "shape": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "ShapeTypes",
        "resolved": "\"default\" | \"edged\" | \"round\"",
        "references": {
          "ShapeTypes": {
            "location": "import",
            "path": "../../utils/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the shape of the input field."
      },
      "attribute": "shape",
      "reflect": false,
      "defaultValue": "'default'"
    },
    "size": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "SizeTypes",
        "resolved": "\"large\" | \"medium\" | \"small\"",
        "references": {
          "SizeTypes": {
            "location": "import",
            "path": "../../utils/types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the size of the input field."
      },
      "attribute": "size",
      "reflect": false,
      "defaultValue": "'medium'"
    }
  }; }
  static get states() { return {
    "text": {}
  }; }
  static get events() { return [{
      "method": "dateChange",
      "name": "dateChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when selected date changes."
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "getInputElement": {
      "complexType": {
        "signature": "() => Promise<HTMLInputElement>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLInputElement>"
      },
      "docs": {
        "text": "Retrieves the native input element.",
        "tags": []
      }
    },
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Set focus on the native input element.",
        "tags": []
      }
    },
    "hide": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the calendar popup if open.",
        "tags": []
      }
    },
    "show": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Opens the calendar popup if closed.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "text",
      "methodName": "handleTextChange"
    }]; }
}
