import { r as registerInstance, c as createEvent, h, H as Host, g as getElement } from './index-9032f50f.js';
import { h as hasSlot } from './slot-0863a9cc.js';

const debounce = (func, wait = 0) => {
  let timer;
  return (...args) => {
    clearTimeout(timer);
    timer = setTimeout(func, wait, ...args);
  };
};

const autocompleteCss = ":host{position:relative;box-sizing:border-box}:host *,:host *:before,:host *:after{box-sizing:inherit}:host{display:inline-block;width:100%}xena-dropdown{display:block}xena-dropdown::part(panel){max-width:80vw}@media screen and (min-width: 768px){xena-dropdown::part(panel){max-width:50vw}}@media screen and (min-width: 992px){xena-dropdown::part(panel){max-width:40vw}}@media screen and (min-width: 1200px){xena-dropdown::part(panel){max-width:30vw}}::slotted([slot=prefix]),::slotted([slot=suffix]){display:flex}";

let Autocomplete = class {
  constructor(hostRef) {
    registerInstance(this, hostRef);
    this.valueChange = createEvent(this, "valueChange", 7);
    this.focused = false;
    this._text = '';
    this._entries = [];
    /**
     * Set the amount of time, in milliseconds, to wait before initiating search after each keystroke.
     */
    this.debounce = 0;
    /**
     * If `true`, the user cannot interact with the input.
     */
    this.disabled = false;
    /**
     * The text in dropdown when no results are found.
     */
    this.noResultText = 'No results';
    /**
     * The property name for the id of the item.
     */
    this.idProperty = 'id';
    /** The autocomplete's label. */
    this.label = '';
    /** The input's help text.  */
    this.helpText = '';
    /** The input's error text. Alternatively, you can use the error-text slot. */
    this.errorText = '';
    /**
     * The property name for the label of the item shown in the dropdown list.
     */
    this.listLabelProperty = 'label';
    /** Indicates whether or not the dropdown is open. You can use this in lieu of the show/hide methods. */
    this.open = false;
    /**
     * If `true`, the user must fill in a value before submitting a form.
     */
    this.required = false;
    /**
     * Defines the shape of the input field.
     */
    this.shape = 'default';
    /**
     * Defines the size of the input field.
     */
    this.size = 'medium';
    /**
     * The selected value of the component.
     */
    this.value = null;
    /**
     * The property name for the label of the item shown in the input field.
     */
    this.valueLabelProperty = 'label';
    /**
     * Whether the dropdown with results should be shown even when there are no results on the components initial focus.
     */
    this.showInitialNoResult = true;
  }
  debounceChanged() {
    this._debounceInput = debounce(ev => {
      ev.stopPropagation();
      this._text = this.input.value;
      if (this.focused && (this.value == null || this._text != this.resolveValueLabel(this.value))) {
        this.search().then(() => this.dropdown.show());
      }
    }, this.debounce);
  }
  /**
   * Sets the datasource to retrieve data from. The datasource is a lambda function which can expect a search query as an argument.
   * In return it must provide a Promise that can deliver an array of data entries.
   * @param datasource
   */
  async setDatasource(datasource) {
    this._datasource = datasource;
  }
  /**
   * Sets focus on the input field.
   */
  async setFocus() {
    this.input.setFocus();
  }
  /**
   * Retrieves the native input element.
   */
  async getInputElement() {
    return this.input.getInputElement();
  }
  /**
   * Closes the autocomplete popup if open.
   */
  async hide() {
    this.dropdown.hide();
  }
  /**
   * Opens the autocomplete popup if closed.
   */
  async show() {
    return this.search().then(() => this.dropdown.show());
  }
  componentWillLoad() {
    this.valueChanged();
    this.handleSlotChange();
  }
  componentDidRender() {
    if (this.list && typeof this.list.getMenuElement === 'function') {
      this.list.getMenuElement().then(menu => menu.style.setProperty('max-height', this.maxDropdownHeight));
    }
  }
  connectedCallback() {
    this.debounceChanged();
    if ('MutationObserver' in window) {
      this.mutationObserver = new MutationObserver(() => this.handleSlotChange());
      this.mutationObserver.observe(this.host, { attributes: false, childList: true, subtree: false });
    }
  }
  disconnectedCallback() {
    var _a;
    (_a = this.mutationObserver) === null || _a === void 0 ? void 0 : _a.disconnect();
  }
  valueChanged() {
    this._text = this.resolveValueLabel(this.value);
    if (this.input) {
      this.input.value = this._text;
    }
  }
  handleSlotChange() {
    this.hasPrefixSlot = hasSlot(this.host, 'prefix');
    this.hasSuffixSlot = hasSlot(this.host, 'suffix');
    this.hasErrorTextSlot = hasSlot(this.host, 'error-text');
  }
  search() {
    if (!this._datasource)
      return Promise.resolve();
    return this._datasource(this._text).then(result => {
      this._entries = result;
      this.list.entries = result;
      this.updateDefaultActiveIndex();
      if (this._entries == null || this._entries.length == 0) {
        return;
      }
      setTimeout(() => {
        try {
          this.dropdown.scheduleUpdate();
        }
        catch (error) {
          /** IGNORE ERROR */
        }
      }, 10);
    });
  }
  handleChange(ev) {
    this._debounceInput(ev);
  }
  handleSelection(ev) {
    ev.stopPropagation();
    this.dropdown.hide();
    const item = ev.detail.item;
    const id = item.value;
    this.value = null;
    if (id != null) {
      this._entries.forEach(e => {
        if (this.resolveId(e) == id) {
          this.value = e;
        }
      });
    }
    this.valueChange.emit(this.value);
  }
  handleFocus() {
    this.focused = true;
    if (this.value == null && !this.open) {
      this.search().then(() => {
        if ((this._entries != null && this._entries.length > 0) || this.showInitialNoResult) {
          this.dropdown.show();
        }
      });
    }
  }
  handleBlur() {
    this.focused = false;
    if (this.input) {
      this.input.value = this._text;
    }
    this.checkIfCleared();
  }
  handleDropdownShow() {
    this.updateDefaultActiveIndex();
    this.open = true;
  }
  updateDefaultActiveIndex() {
    const activeIndex = this._entries.length > 0 ? 0 : undefined;
    this.list.defaultActiveIndex = activeIndex;
  }
  handleDropdownAfterHide() {
    this.open = false;
    this.checkIfCleared();
    this.valueChanged();
  }
  checkIfCleared() {
    if ((!this._text || this._text.trim() === '') && this.value != null && !this.open) {
      this.value = null;
      this.valueChange.emit(this.value);
    }
  }
  resolveId(entry) {
    return entry[this.idProperty];
  }
  resolveValueLabel(entry) {
    if (entry == null)
      return '';
    if (typeof this.valueLabelProperty === 'function') {
      return this.valueLabelProperty(entry);
    }
    else {
      return entry[this.valueLabelProperty];
    }
  }
  handleKeyDown(event) {
    // Select active element when tabbing out of autocomplete
    if (['Tab', 'Enter'].includes(event.key)) {
      this.list.getMenuElement().then(el => el.selectActiveMenuElement());
      if (event.key === 'Tab' && this._entries.length === 0) {
        this.hide();
      }
    }
  }
  handleValueChanged(event) {
    event.stopPropagation();
  }
  render() {
    return (h(Host, null, h("xena-dropdown", { ref: el => (this.dropdown = el), hoist: true, triggerClickAction: 'none', noToggleKey: true, onXnShow: _ev => this.handleDropdownShow(), onXnAfterHide: _ev => this.handleDropdownAfterHide() }, h("xena-input", { label: this.label, helpText: this.helpText, errorText: this.errorText, ref: el => (this.input = el), disabled: this.disabled, slot: "trigger", placeholder: this.placeholder, value: this._text, onFocus: _ev => this.handleFocus(), onBlur: _ev => this.handleBlur(), onXnInput: _ev => this.handleChange(_ev), onValueChange: _ev => this.handleValueChanged(_ev), shape: this.shape, size: this.size, onKeyDown: _ev => this.handleKeyDown(_ev), required: this.required }, this.hasPrefixSlot ? (h("span", { slot: "prefix", part: "prefix" }, h("slot", { name: "prefix" }))) : (''), this.hasSuffixSlot ? (h("span", { slot: "suffix", part: "suffix" }, h("slot", { name: "suffix" }))) : (''), this.hasErrorTextSlot ? (h("span", { slot: "error-text", part: "error-text" }, h("slot", { name: "error-text" }))) : ('')), h("xena-autocomplete-list", { ref: el => (this.list = el), noResultText: this.noResultText, entries: this._entries, onXnSelect: ev => this.handleSelection(ev), idProperty: this.idProperty, labelProperty: this.listLabelProperty, iconProperty: this.listIconProperty }))));
  }
  get host() { return getElement(this); }
  static get watchers() { return {
    "debounce": ["debounceChanged"],
    "value": ["valueChanged"]
  }; }
};
Autocomplete.style = autocompleteCss;

export { Autocomplete as xena_autocomplete };
