# Data presentation

Presenting data in a natural way for the user is crucial to make the interface comprehensible.

## Data containers

### Cards

A [card](/components/card) is a container for information and actions related to a single concept or object. Cards can help highlight and structure information. While they're very flexible, it's important to use them consistently.

Cards should never contain a whole view, like [panels](/components/panel) can do.

### Key-value pairs

The structure of keys and values are most often used in "read mode" of data. A component specifically for this pattern is still in pipeline, but the [statistic component](/components/statistic) might be useful to some extent.

![An order from EG Xena with many key-value pairs, like customer, department, references, etc.][def5]

In raw HTML, a good element to use in this scenario is [description list](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/dl):

```html
<dd>
    <dt>Partner</dt>
    <dd>Knud Peter Andresen (<a href="#" title="Partner details">12348</a>)</dd>
</dl>
```

### Lists

[Lists](/components/list) are good to browse information or select an item within a group. Content is very simple and to the point. Use it to view description and detailed information, present features, or as links to dive deeper.

![A multi-page list of notifications.][def1]

!> The difference between a list and a table is often the need for headings. Do not use a table if a list is sufficient.

### Tables

[Tables](/components/table) are used to organize and display data efficiently. Tables has headers, columns, rows and cells. When designing a table, keep the following guidelines in mind:

- Column headers should be short, like "Name" and "Price". Avoid half and full sentences, like "Users' full name" and "Is sold for".
- All numbers should be right aligned.

![A table with 8 columns and three rows.][def4]

#### Expandable rows

Expandable rows are useful for presenting large amounts of data in a small space. Use the expanded row for supplementary information or data that needs additional query time.

![A table with unfolded row, revealing additional rows.][def2]

The expanded row can either continue the parent table (as above), or contain another table or data (as below).

![A table with unfolded row, revealing additional data.][def3]

## Data types

### Numbers

Numbers should be formatted in the pattern for the users' language setting. For example, when it comes to thousand separators and decimals.

| Culture   | Number formatting |
| --------- | ----------------: |
| Danish    |        123.456,78 |
| English   |        123,456.78 |
| Norwegian |        123 456,78 |

For currencies and units, you can add the abbreviation after the number. There should be a space between value and unit abbreviation. For currency, use the international codes.

| Type       |                         Example |
| ---------- | ------------------------------: |
| Currency   |                    1,000.00 DKK |
| Length     |                       120.00 cm |
| Percentage |                         25.00 % |
| Size       | 45 m<sup><small>2</small></sup> |
| Weight     |                        12.50 kg |

### Date

Dates should be either formatted in machine readable format or the format matching the user's language settings. Below are examples of the differences:

| Culture         | Date format  | Example                                          |
| --------------- | ------------ | ------------------------------------------------ |
| Machine         | YYYY-MM-DD   | <time datetime="2024-01-31">2024-01-31</time>    |
| Danish          | DD.MM.YYYY   | <time datetime="2024-01-31">31.01.2024</time>    |
| English         | MM/DD/YYYY   | <time datetime="2024-01-31">01/31/2024</time>    |
| Norwegian       | DD.MM.YYYY   | <time datetime="2024-01-31">31.01.2024</time>    |
| Danish, long    | D. MMM. YYYY | <time datetime="2024-01-31">31. jan. 2024</time> |
| English, long   | MMM D, YYYY  | <time datetime="2024-01-31">Jan 31, 2024</time>  |
| Norwegian, long | D. MMM. YYYY | <time datetime="2024-01-31">31. jan. 2024</time> |

Also, see how to [markup dates and time](https://developer.mozilla.org/en-US/docs/Web/HTML/Element/time) in HTML.

#### Date range

Use an `&ndash;` between start and end dates.

| Version | Example                                                                                                                        |
| ------- | ------------------------------------------------------------------------------------------------------------------------------ |
| Short   | <time datetime="P365D"><time>2021-01-01</time> – <time>2021-12-31</time></time>                                                |
| Long    | <time datetime="P365D"><time datetime="2021-01-01">Jan 1, 2021</time> – <time datetime="2021-12-31">Dec 31, 2021</time></time> |

### Time

Always use colon as separator. Time should only include seconds if relevant for the view.

| Version      | Example               |
| ------------ | --------------------- |
| Short format | <time>09:00</time>    |
| Long format  | <time>09:00:00</time> |
| 24-hour      | <time>14:00</time>    |
| 12-hour      | <time>2:00 PM</time>  |

#### Time range

Use an `&ndash;` between start and end times.

Format: HH:MM–HH:MM

Example: <time datetime="PT8H30M">08:00–16:30</time>

### Duration

A representation of days, hours and minutes. Omit values that don't apply. The abbreviations should match the user's UI language.

Format: Xd Yh Zm

Example: <time datetime="P1DT2H43M">1<abbr title="day(s)">d</abbr> 2<abbr title="hour(s)">h</abbr> 43<abbr title="minute(s)">m</abbr></time>

See the [duration string documentation](https://www.w3.org/TR/2012/WD-html5-20121025/common-microsyntaxes.html#valid-duration-string) on how to markup a duration.

### Relative time

Relative time can be easier to grasp when used for recent events. Here is an example system:

| When                   | Example                                                      |
| ---------------------- | ------------------------------------------------------------ |
| Next date              | <time datetime="2024-01-02T12:00:00">Tomorrow, 12:00</time>  |
| In less than 24 hours  | <time datetime="2024-01-02T12:00:00">Today at 12:00</time>   |
| In less than 1 hour    | <time datetime="2024-01-02T12:00:00">In X minutes</time>     |
| Less than 1 minute ago | <time datetime="2024-01-02T12:00:00">Just now</time>         |
| Less than 1 hour ago   | <time datetime="2024-01-02T12:00:00">X minutes ago</time>    |
| Less than 24 hours ago | <time datetime="2024-01-02T12:00:00">X hours ago</time>      |
| 24 to 48 hours ago     | <time datetime="2024-01-02T12:00:00">Yesterday, 12:00</time> |

## Time and date abbreviations

### Months

| Name      | Abbreviation |
| --------- | ------------ |
| January   | Jan          |
| February  | Feb          |
| March     | Mar          |
| April     | Apr          |
| May       | May          |
| June      | Jun          |
| July      | Jul          |
| August    | Aug          |
| September | Sept         |
| October   | Oct          |
| November  | Nov          |
| December  | Dec          |

### Days

| Name      | Abbreviation 1 | Abbreviation 2 | Abbreviation 3 |
| --------- | -------------- | -------------- | -------------- |
| Monday    | Mon            | Mo             | M              |
| Tuesday   | Tue            | Tu             | T              |
| Wednesday | Wed            | We             | W              |
| Thursday  | Thu            | Th             | T              |
| Friday    | Fri            | Fr             | F              |
| Saturday  | Sat            | Sa             | S              |
| Sunday    | Sun            | Su             | S              |

### Duration units

| Unit      | Abbreviation |
| --------- | ------------ |
| Year(s)   | N/A          |
| Month(s)  | N/A          |
| Day(s)    | d            |
| Hour(s)   | h            |
| Minute(s) | m            |
| Second(s) | s            |

?> **Tip:** On the EG Xena website we use [JavaScript Intl](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/Intl/DateTimeFormat) to format dates automatically.

[def1]: ../assets/patterns/data_list.jpg
[def2]: ../assets/patterns/data_tablerows.jpg
[def3]: ../assets/patterns/data_tablerows2.jpg
[def4]: ../assets/patterns/data_table.jpg
[def5]: ../assets/patterns/data_keyvalue.jpg
